#!/usr/bin/perl -w
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 
# Copyright (C) 2000,2001 Mandrakesoft
# Author Renaud Chaillat <rchaillat@mandrakesoft.com>
#
# $Id: config-test.pl,v 1.2 2002/12/04 15:26:15 philippe Exp $
#

# Script to perform some tests on current transaction variables and values
# Do *not* perform write operations here since we use an existing transaction

# do not print messages in this program, since
# most of the time its output will be processed
# by a calling program

# Command line version

# Error exit codes:
# -1: variable exists but is empty
# -2: variable does not exist
# -3: 'parachute' exit

# Matching exit codes:
# 0: variable does not match any pattern, or no pattern was given, 
#    but the variable is not empty
# 1: the first pattern matched
# 2: the second pattern matched
# and so on...

use strict;
no strict 'refs';
use lib qw(/usr/lib/naat/);
use backend::config;
#use backend::util; 

#-------------------------------------------------------------
# parse configuration file and store path to database file
#-------------------------------------------------------------

# configuration filename
# (database for system parameters)
my $config_filename = "";


open (CONFIG, "/etc/naat/backend.conf") or die "Can't open config file: $!";
while (<CONFIG>) {
    chomp;                  # no newline
    s/#.*//;                # no comments
    s/^\s+//;               # no leading white
    s/\s+$//;               # no trailing white
    next unless length;     # anything left?
    my ($var, $value) = split(/\s*=\s*/, $_, 2);
    $value =~ s/\"//g;      # remove "s
    if ( $var =~ /CONFIGURATION_FILE/ ) { 
	$config_filename = $value;
    }
}
close (CONFIG);

unless ( defined ($config_filename) ) { 
    $config_filename = "/var/lib/naat/configuration";       
}


#-------------------------------------------------------------
# Read arguments
#-------------------------------------------------------------
#
# 

# Check if any argument is given
if ( scalar (@ARGV) == 0 ) {
    print STDERR "Wrong number of arguments :\n"; #$Usage; 
    print STDERR "config-test transaction_name variable_name [pattern] \n"; #$Usage; 
    exit(-3);
}

# ARG[0] is the transaction name
my $transaction = shift @ARGV;

if (scalar (@ARGV) == 0) {
    print STDERR "No variable name given after the transaction name \n";
    exit (-3);
}


# arg 1 is a variable name
# arg 2, 3, ... is a value/regexp
my $varname = shift @ARGV;
# capitalize (all backend variables *must* start with a capital letter)
$varname = "\u$varname";
my @patterns = @ARGV;

# Transaction conf parameters
my %conf;
tie %conf, 'backend::config', "$config_filename.working.$transaction";


#-------------------------------------------------------------
# Only test existence and definition of a variable
#-------------------------------------------------------------

unless ( defined ($conf{$varname}) ) { 
    print STDERR "$varname does not exist in configuration file \n";
    exit (-2); 
}
unless ( $conf{$varname} =~ /\S+/ )  { 
    print STDERR "$varname exists but is empty in configuration file \n";
    exit (-1); 
}

# if no pattern is given print the value to stderr and exit successfully
if ( scalar (@patterns) == 0 ) { 
    print STDERR "$varname has value $conf{$varname} \n";
    exit (0);
}


#-------------------------------------------------------------
# Match pattern
#-------------------------------------------------------------
#

# Test if one or more variables match patterns
my $test_result = 0;
my $matching_pattern = 0; # just to make the code clear
my $counter = 0;
print STDERR "$varname with value $conf{$varname} ";
while (@patterns) {
    my $pattern = shift @patterns;
    $counter++;
    if ( $conf{$varname} =~ m{ $pattern }ix ) {
	print STDERR "matches the pattern $pattern \n";
	$matching_pattern = $counter;
	exit ($matching_pattern); 
    } 
}

print STDERR "does not match any pattern \n";
exit (0);


# Example : IP Address
# m{
#    ^  ( \d | [01]?\d\d | 2[0-4]\d | 25[0-5] ) 
#    \. ( \d | [01]?\d\d | 2[0-4]\d | 25[0-5] )
#    \. ( \d | [01]?\d\d | 2[0-4]\d | 25[0-5] )
#    \. ( \d | [01]?\d\d | 2[0-4]\d | 25[0-5] )
#    $
# }x





