#!/usr/bin/perl -w
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2000,2001 Mandrakesoft
# Author Renaud Chaillat <rchaillat@mandrakesoft.com>
#
# $Id: config-wrapper.pl,v 1.36 2003/11/19 21:47:10 florin Exp $
#

# Wrapper for configuration file
# => Implements Set and Get on variables
# => Trigger events

# Exit codes:
# 0: normal exit
## obsolete 1: variable exists but is empty (! DO NOT EXIT ANYMORE TO ALLOW MULTIPLE GETS)
## obsolete 2: variable does not exist (removed to not exit from multiple gets)
# 3: 'parachute' exit
# 4: master file was modified externally during the current transaction
#    (i.e. another commit took place)

# do not print messages to stdout in this

# ------------------------------------------------------------------------------
# ChangeLog :
# Change the list add function :
#    - Now, We don't have to passed an item
#    - if more than one args is passed, the list will be in the form 1(attr=value...),2(...)
#      else, the list will be in the form value1,value2,value3
# Change the list del function :
#    - list del by attributes
#
# Add list del key :
#    - list del by keys


# Command line version (not a module)

use strict;
# use File::Basename;
#-- If you don't want to use tie, comment out these two lines --
#-- and see file_load ()
# changeLog : This doesn't work with long list... so don't do it.
use lib qw(/usr/lib/naat/);
use backend::config;

# TEMPORARY
# use Data::Dumper;


my $version = 0.5;  # 0.4: virtual vars, 0.5: multiple get

# convenience variables for true and false
my $false = 0;
my $true = 1;


my $NULL_ITEM = "---\n";
my $NULL_VAR_NAME = "-*-\n";

# hash table to store parsed values in configuration files
my %master_vars = ();
my %transaction_vars = ();

my %trigger = ();


my $Usage =<<"End_of_Usage;";
Usage: config-wrapper transaction_name <-i | -g | -m | -s | -la | -ld | -a | -c | -lm> 
       config-wrapper <--backup | --restore | --help | --version>

  * Transactions operations:

  -i --init          Initialize the transaction

  -g --get           Prints one variable content or all variable definitions/values
                     e.g. --get <parameter-name> [<arguments>]
                     	  --get all (or --get)

  -m --mget          Prints multiple variables content in a comma separated string
                     (only with normal variables, not with proxy variables)
                     e.g. --mget <param1> <param2> ...

  -s --set           Sets a variable
                     e.g. --set <param1> <arg1> [<arg2> <arg3> ...]
		             You can use multiple -s options on the same command line
		             e.g. --set <param1> <arg1> [<arg2>...] --set <param2> <arg2> ...

  -ls --listset      Change an attribute of a variable
                     e.g. --listset <param1> <attr1>=<value1> [ <attr2>=<value2> ...]

  -lg --listget      Print attributes from item list from var
                     e.g. --listget <parameter-name> <item> [<attribute>]

  -lm --listmget      Print all the attributes of the same type from var
                     e.g. --listmget <parameter-name> <item> [<attribute>]

  -la --listadd      Add one or several elements to a list-type variable: var=item1,item2,...
                     An element can have attributes: item1 (attr1=value1 attr2=value2 ...),item2 ...
                     e.g. --listadd <param>,<attr1>,<value1>,<attr2>,<value2>,...

  -lainc --listaddinc Add one or several elements to a list-type variable: var=item1,item2,...
  		     and increment the item1 variable
                     An element can have attributes: item1 (attr1=value1 attr2=value2 ...),item2 ...
                     e.g. --listadd <param>,<attr1>,<value1>,<attr2>,<value2>,...

  -lains --listaddins Add one or several elements to a list-type variable: var=item1,item2,...
  		     and increment ALL the following itemx variables
                     An element can have attributes: item1 (attr1=value1 attr2=value2 ...),item2 ...
                     e.g. --listadd <param>,<attr1>,<value1>,<attr2>,<value2>,...

  -ld --listdel      Del one or several elements from a list-type variable
                     You must specified all attr and their values to the param be cleaned
                     e.g --listdel <param> <attr1> <value1> <attr2> <value2>..., <attr'1> <value'1>...
                     If one attr (value or miss) of the list is'nt correct, the element won't be delete.
                     (Don't use it directly by hand... better to use script than by hand)

  -ldattr --listdelattr Del one or several elements from a list-type variable
                     You must specified all attr and their values to the param be cleaned
                     e.g --listdel <param> <attr1> <value1> <attr2> <value2>..., <attr'1> <value'1>...
                     If one attr (value or miss) of the list is'nt correct, the element won't be delete.
                     (Don't use it directly by hand... better to use script than by hand)

  -ldk --listdelkeys Del one or several elements from a list-type variable
                     If one or several items are given, only these part of item's list will be removed
                     e.g. --listdel <param> <item>,<item2>,...

  -a --abort         Abort the transaction

  -c --commit        Commit the transaction
                     (apply changes on the master configuration)
                     Returns an error if configuration has changed during the transaction.

  -f --force         Commit the transaction without any check.

  * Backup and restore operations:

  -b --backup        Backup the configuration file
                     e.g. --backup 
                     	  --backup <destination_filename_with_path>
  -r --restore       Restore system configuration
                     e.g. --restore
                     	  --restore <config_backup_filename_with_path>
  -u --update        Update the whole system by triggering *all* events
                     (i.e. apply all configuration values)
  -h --help          Prints this page to standard output.
  -v --version       Prints version information to standard output.
End_of_Usage;

##############################################################

#-------------------------------------------------------------
# parse backend configuration file and store variables and values
#-------------------------------------------------------------
my %backend_conf = (); 
open (CONFIG, "/etc/naat/backend.conf") or die "Can't open config file: $!";
while (<CONFIG>) {
    chomp;                  # no newline
    s/#.*//;                # no comments
    s/^\s+//;               # no leading white
    s/\s+$//;               # no trailing white
    next unless length;     # anything left?
    my ($var, $value) = split(/\s*=\s*/, $_, 2);
    $value =~ s/\"//g;      # remove "s
    $backend_conf{$var} = $value;
}
close (CONFIG);

# master configuration filename
# (database for system parameters)
my $config_filename = "";
if ( defined ($backend_conf{"CONFIGURATION_FILE"}) ) { 
    $config_filename = $backend_conf{"CONFIGURATION_FILE"};
} else {
    $config_filename = "/var/lib/naat/configuration";
}

# default scripts dir
my $scripts_default_dir = "";
if ( defined ($backend_conf{"SCRIPTS_DEFAULT_DIR"}) ) { 
    $scripts_default_dir = $backend_conf{"SCRIPTS_DEFAULT_DIR"};
} else {
    print "Scripts default dir set to default: /usr/share/naat/scripts\n";
    $scripts_default_dir = "/usr/share/naat/scripts";
}

#-------------------------------------------------------------
# Actual work
#-------------------------------------------------------------
#
# Read command line options (set or get, parameter and arguments)
# and process.
# Set some global variables relative to the transaction.
#

# Check if any argument is given
if ( scalar (@ARGV) == 0 ) {
    print STDERR $Usage;
    exit(3);
}


my $transaction_name = "trans_".`date +%s`;
chomp ($transaction_name);
$transaction_name .= "_".$$;


# Verify arguments a little and get the transaction name if needed
#
### TODO: USE POPT !!! ###
#
my $args = "@ARGV";
print STDERR "Config-wrapper called with arguments: $args \n";

if ( ($args =~ /\s+-i\s+|\s+--init\s+|\s+-s\s+|\s+--set\s+|\s+-ls\s+|\s+--listset\s+|\s+-g\s+|\s+--get\s+|\s+-m\s+|\s+--mget\s+|\s+--listmget\s+|\s+-c\s+|\s+--commit\s+|\s+-f\s+|\s+--force\s+|\s+-a\s+|\s+--abort\s+|\s+-la\s+|\s+--listadd\s+|\s+--lainc\s+|\s+--listaddinc\s+|\s+--lains\s+|\s+--listaddins\s+|\s+-ld\s+|\s+--listdel\s+|\s+-ldattr\s+|\s+--listdelattr\s+|\s+-ldk\s+|\s+--listdelkeys\s+|\s+-lg\s+|\s+--listget\s+|s+-d\s+/) &&
     ($args =~ /\s+-b\s+|\s+--backup\s+|\s+-r\s+|\s+--restore\s+|\s+-u\s+|\s+--update\s+|\s+-v\s+|\s+--version\s+|\s+-h\s+|\s+--help\s+/) ) {
	print STDERR "Parameters mismatch\n";
	print STDERR $Usage;
	exit(3);
}
if ( $ARGV[0] !~ /^-.*/ ) {  # First arg is the transaction name
    $transaction_name = shift @ARGV;
    if ( @ARGV && 
	 ($args = "@ARGV") =~ /\s+-b\s+|\s+--backup\s+|\s+-r\s+|\s+--restore\s+|\s+-u\s+|\s+--update\s+|\s+-v\s+|\s+--version\s+|\s+-h\s+|\s+--help\s+/ ) {
	print STDERR "Operation not permitted with a transaction\n";
	print STDERR $Usage;
	exit(3);
    }
}
else {     # First arg is *not* the transaction name
    if ( $args  =~ /\s+-i\s+|\s+--init\s+|\s+-s\s+|\s+--set\s+|\s+-ls\s+|\s+--listset\s+|\s+-g\s+|\s+--get\s+|\s+-m\s+|\s+--mget\s+|\s+--listmget\s+|\s+-c\s+|\s+--commit\s+|\s+-f\s+|\s+--force\s+|\s+-a\s+|\s+--abort\s+|\s+-la\s+|\s+--listadd\s+|\s+--lainc\s+|\s+--listaddinc\s+|\s+--lains\s+|\s+--listaddins\s+|\s+-ld\s+|\s+--listdel\s+|\s+-ldattr\s+|\s+--listdelattr\s+|\s+-ldk\s+|\s+--listdelkeys\s+|\s+-lg\s+|\s+--listget\s+|\s+-d\s+/ ) {
	print STDERR "Operation needs a transaction name\n";
	print STDERR $Usage;
	exit(3);
    }
}


my $trans_file        = "${config_filename}.working.${transaction_name}";
my $trans_backup_file = "${config_filename}.backup.${transaction_name}";


# Sanity check
if (! defined($ARGV[0]) ) {
    print STDERR "No argument found, either you forgot the transaction name,\n";
    print STDERR "or did not give any option after the transaction, \n";
    print STDERR "or did not give any argument at all; \n";
    print STDERR "transaction: $transaction_name .\n";
    exit (3);
}

# Transactions operations
if ( $ARGV[0] eq "--init"       || $ARGV[0] eq "-i" ) {
    config_init_transaction ();
}
elsif ( $ARGV[0] eq "--get"     || $ARGV[0] eq "-g" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction: $transaction_name !!!\n";
	config_init_transaction ();
    }

    config_load_transaction ();

    if ( scalar (@ARGV) == 1 &&  $ARGV[0] =~ /^all/ ) {
	config_get_all ();
    }
    elsif ( scalar (@ARGV) >= 1 ) {
#	config_get (@ARGV);

	# parse command line to separate each -g option
	my @arguments = ();
	while ( (@ARGV) ) {
	    while ( (@ARGV) && 
		    (my $argument = shift @ARGV) !~ /^-g|^--get/  ) {
		push (@arguments,$argument);
	    }
	    if ( (@arguments) ) { # Avoid pb with "--get --get" !
		config_get (@arguments);
	    }
	    @arguments = ();
	}
    }
    else {
	config_get_all ();
    }
}
elsif ( $ARGV[0] eq "--mget"     || $ARGV[0] eq "-m" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction: $transaction_name !!!\n";
	config_init_transaction ();
    }

    config_load_transaction ();

    config_multiple_get (@ARGV);
}
elsif ( $ARGV[0] eq "--listmget"     || $ARGV[0] eq "-lm" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction: $transaction_name !!!\n";
	config_init_transaction ();
    }

    config_load_transaction ();

    config_list_multiple_get (@ARGV);
}
elsif ( $ARGV[0] eq "--set"     || $ARGV[0] eq "-s" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    # parse command line to separate each -s option
    my @arguments = ();
    while ( (@ARGV) ) {
	while ( (@ARGV) && 
		(my $argument = shift @ARGV) !~ /^-s|^--set/  ) {
	    push (@arguments,$argument);
	}
	config_set (@arguments);
	@arguments = ();
    }
}
elsif ( $ARGV[0] eq "--listset"     || $ARGV[0] eq "-ls" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    config_load_transaction ();

    config_list_set (@ARGV);
}
elsif ( $ARGV[0] eq "--listadd"     || $ARGV[0] eq "-la" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    config_load_transaction ();

    config_list_add (@ARGV);
}
elsif ( $ARGV[0] eq "--listaddinc"     || $ARGV[0] eq "-lainc" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    config_load_transaction ();

    config_list_addinc (@ARGV);
}
elsif ( $ARGV[0] eq "--listaddins"     || $ARGV[0] eq "-lains" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    config_load_transaction ();

    config_list_addins (@ARGV);
}
# delete list by attributes
elsif ( $ARGV[0] eq "--listdel"     || $ARGV[0] eq "-ld" ) {
    shift @ARGV;

    config_load_transaction ();

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    config_list_del (@ARGV);
}

# delete list by attributes
elsif ( $ARGV[0] eq "--listdelattr"     || $ARGV[0] eq "-ldattr" ) {
    shift @ARGV;

    config_load_transaction ();

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    config_list_del_attr (@ARGV);
}
# delete list by key(s)
elsif ( $ARGV[0] eq "--listdelkeys"     || $ARGV[0] eq "-ldk" ) {
    shift @ARGV;

    config_load_transaction ();

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    config_list_del_keys (@ARGV);
}

elsif ( $ARGV[0] eq "--listget"     || $ARGV[0] eq "-lg" ) {
    shift @ARGV;

    # test if this transaction has already been initiated
    unless ( -f $trans_file ) {
	print STDERR "!!! Initializing transaction !!!\n";
	config_init_transaction ();
    }

    config_load_transaction ();

    config_list_get (@ARGV);
}
elsif ( $ARGV[0] eq "--abort"   || $ARGV[0] eq "-a" ) {
    config_abort_transaction ();
}
elsif ( $ARGV[0] eq "--commit"  || $ARGV[0] eq "-c" ) {

    # TODO: check if transaction has been initiated
    # (if initiated and nothing changed, apply will
    #  do nothing)

    config_commit_transaction ();
}

elsif ( $ARGV[0] eq "--force"   || $ARGV[0] eq "-f" ) {

    # TODO: check if transaction has been initiated
    # (if initiated and nothing changed, apply will
    #  do nothing)

    config_apply_transaction ();
}

# Backup and restore operations
elsif ( $ARGV[0] eq "--backup"  || $ARGV[0] eq "-b" ) {
    shift @ARGV;
    config_backup (@ARGV);
}
elsif ( $ARGV[0] eq "--restore" || $ARGV[0] eq "-r" ) {
    shift @ARGV;
    config_restore (@ARGV);
}
elsif ( $ARGV[0] eq "--update" || $ARGV[0] eq "-u" ) {
    config_apply_all ();
}
elsif ( $ARGV[0] eq "--help"    || $ARGV[0] eq "-h" ) {
    print STDERR $Usage;
    exit(0);
}
elsif ( $ARGV[0] eq "--version" || $ARGV[0] eq "-v" ) {
    print STDERR "$version\n";
    exit(0); }
else
{
    print STDERR "Error: Unrecognized option '$ARGV[0]'.\n\n";
    print STDERR $Usage; exit(3);
}

# Normal exit case, just for clarity
exit (0);


#-------------------------------------------------------------
# Transaction Functions
#-------------------------------------------------------------
# config_init_transaction
# config_load_transaction
# config_abort_transaction
# config_commit_transaction
# config_apply_transaction
# config_set
# config_list_set
# config_list_add
# config_list_addinc
# config_list_addins
# config_list_del
# config_list_del_keys
# config_list_get # get attributes and values of on item in a list
# config_list_multiple_get # get all the attributes values of a composed variable
# config_get
# config_get_all
# config_diff
# perform_action
# execute_direct_scripts
# add_attr_value
# del_attr_value
# build_hash_from_list
# stringify_hash


# Init the transaction by :
# - creating a transaction working copy of the configuration file.
# - creating a transaction backup copy of the configuration file,
#   which will be used to check if commits from other transactions
#   occured.
# Note: An init is not strictly necessary (a get or set will init
# anyway if transaction does not exists) but it ensures we begin
# with clean transaction files (exact copies of the master file).
# (i.e. with init we do not *continue* a previous transaction with
#  the same id, we *reset* it)
sub
config_init_transaction
{
#    print STDERR "Transaction initiated: $trans_file";

    # transaction working copy of the configuration file
    [ -f $trans_file ] && unlink ($trans_file);
    system ("cp", $config_filename, $trans_file);

    # transaction backup copy of the configuration file
    [ -f $trans_backup_file ] && unlink ($trans_backup_file);
    system ("cp", $config_filename, $trans_backup_file);
}

# Load the transaction copy of the configuration file into 
# a Hash
sub
config_load_transaction
{

    if (scalar (%transaction_vars)) { %transaction_vars = (); }

    %transaction_vars = file_load ($trans_file);

    return;
}


sub
config_abort_transaction
{
    my $file1 = $trans_file;
    my $file2 = $trans_backup_file;
    my $file3 = "${trans_backup_file}.temp";

    # cleanup transaction files
    [ -f $file1 ] && unlink ($file1);
    [ -f $file2 ] && unlink ($file2);
    [ -f $file3 ] && unlink ($file3);
}

sub
config_commit_transaction
{
    # compare transaction config file to master config to see
    # if other commits occured 
    my %diff = compute_diff ($config_filename, $trans_backup_file);

    if ( scalar (%diff) ) {
	print STDERR "Commits have occured in other transactions.\n";
	print STDERR "Use --force option to override changes.\n";
	exit (4);
    }

    # apply changes
    config_apply_transaction ();
}

# Apply changes without any checks
# TODO : lock ?
sub
config_apply_transaction
{
    my $backupfile = "${config_filename}.backup";

    # Compute changes to apply to the master config file
    my %changes = config_diff ();
    print STDERR scalar(%changes);
    unless ( scalar (%changes) ) {
	print STDERR "Nothing has changed, so no system change is triggered.\n";
	print STDERR "==> You may use --update if you want to apply *all* values.\n";
	print STDERR "Alternatively you may use event-handler.pl directly, \n";
	print STDERR "using parameter names as arguments (i.e. events to trigger).\n";
	exit (3);
    }
    ## DEBUG #
    foreach (keys %changes) {
	print STDERR "$_ parameter has a new value: $changes{$_}\n";
    }
    # DEBUG ##

    # Create a brand new backup of the master config file
    [ -f $backupfile ] && unlink ($backupfile);
    system ("cp", $config_filename, $backupfile);

    # Update the master config file
    config_update_master_file ();

    # Find scripts to launch directly (i.e. virtual/proxy variables)
    execute_direct_scripts (%changes);
    
    # Trigger necessary events to update the system
    my @events = compute_events (%changes);
    if ( scalar (@events) ) {   # if there are events to trigger
	print STDERR "**********************\n";
	print STDERR "*** TRIGGER EVENTS ***\n";
	print STDERR "*** @events ***\n";
	print STDERR "**********************\n";
	system ("sudo", "/usr/bin/event-handler.pl", $transaction_name, @events);
    }

    # Clean up files
    # TODO: check if previous copy ops went well before deleting
    config_abort_transaction ();
}


# Isolate static variables
sub
compute_events
{
    my %pairs = @_;
    my @events = ();

    while ( my ($key, $value) = each (%pairs) ) {
	if ( $value !~ /(set: | get: | trigger: )/ ) {
	    push @events, ($key);
	}
    }

    return @events;
}


# Copy the transaction config file as the master config file, 
# but strip values appended to virtual/proxy variables
sub
config_update_master_file
{
    open (INFILE, $trans_file) or do {
	print STDERR "Transaction file does not appear to exist !\n";
	print STDERR "** Cancelling transaction commit... **";
	exit (3);
    };

    open (OUTFILE, "> ${config_filename}.update") or do {
	print STDERR "** Creating the update file for master configuration was impossible. **\n";
	exit (3);
    };

    while (<INFILE>) {
	my $line = $_;		# preserve line if we don't know it
	chomp;			# no newline
	s/#.*//;		# no comments
	s/^\s+//;		# no leading white
	s/\s+$//;		# no trailing white
	next unless length;	# anything left?

	# get variable from file and its value
	my ($var_name, $var_value) = split (/\s*=\s*/, $_, 2);

	# parse to see if we must strip this line
	if ( $var_value =~ /(set: | get: | trigger: )/ ) {
	    $var_value =~ s/(\'.*\').*/$1/;
	    print OUTFILE "$var_name=$var_value\n";
	} else {
	    print OUTFILE "$line";	# write the line untouched
	}
    }

    close (OUTFILE);
    close (INFILE);

    # TODO: Check if we can safely overwrite the master config file
    [ -f $config_filename ] && unlink ($config_filename);
    system ("cp", "${config_filename}.update", $config_filename);
    [ -f "${config_filename}.update" ] && unlink ("${config_filename}.update");

}

sub
execute_direct_scripts
{
    my %pairs = @_;

    while ( my ($key, $value) = each (%pairs) ) {
	print STDERR "In exec direct: ", $key, " : ", $value, "\n";
	if ( $value =~ /set:/ ) {
	    perform_action ("set", $value);
	} else {
	    print STDERR "In exec direct: Not a virtual/proxy variable, skip. \n";
	}
    }
}

# Updates one variable for the current transaction
# Note: if the variable is a proxy (its content is a script, i.e. virtual), 
#       just append the arguments to the content; the script 
#       will be executed during the commit
sub
config_set
{
    my $var_to_set = shift @_;
    my $infile = "${trans_file}.temp";
    my $outfile = "> ${trans_file}";
    my @trigger_command = ();

    unless (defined ($var_to_set)) { 
	print STDERR "\nEmpty variable name ???\n";
	return; # should report error case
    }

    # capitalize
    $var_to_set = "\u$var_to_set";

    # backup transaction configuration file into a temporary file
    [ -f $infile ] && unlink ($infile);
    system ("cp", $trans_file, $infile);

    ### update configuration file with values in %changes #
    open (INFILE, $infile);
    open (OUTFILE, $outfile);

    ## DEBUG #
    print STDERR "\n*** APPLY ***\n";
    print STDERR "Change: $var_to_set = ", join (" ", @_),"\n";
    print STDERR "\n";
    # DEBUG ##

    while (<INFILE>) {
      my $line = $_;		# preserve line if we don't know it
      chomp;			# no newline
      s/#.*//;		 	# no comments
      s/^\s+//;			# no leading white
      s/\s+$//;			# no trailing white
      next unless length;	# anything left?

      # get variable from file and its value
      my ($var_name, $var_value) = split(/\s*=\s*/, $_, 2);
      $var_value =~ s/\s+$// if ($var_value);	# remove trailing spaces
      # parse to see if we have to update this line
      if ($var_to_set eq $var_name) { # we found the variable to set
	#	    s/\s/ /g;  # change tabs and internal newlines into spaces
	my $args = "";
	#	    foreach (@_) { $args .= "\"$_\" "; }; 
	$args = join (' ', @_);
	print STDERR "======== CONFIG SET ARGS: **${args}** \n";
	if ( $var_value =~ /set:/ ) {
	  $var_value =~ s/\s+$//;
	  print OUTFILE "$var_name=$var_value $args\n";
	} elsif ( $var_value =~ /trigger:/ ) {
	  $var_value =~ s/\s+$//;
	  push( @trigger_command, "$transaction_vars{$var_name}=$var_value $args" );
	  print OUTFILE "$var_name=$var_value\n";
#	  print OUTFILE "$var_name=$
	} else {
	  print OUTFILE "$var_name=$args\n";
	}
      } else {
	print OUTFILE "$line";	# write the line untouched
      }
    }


    #Here trigger command to execute
    foreach( @trigger_command ) {
      print "trigger => $_\n";
      perform_action( "trigger", $_ );
    }


    close (OUTFILE);
    close (INFILE);
    # file updated ###

    # clean up
    [ -f "${trans_file}.temp" ] && unlink ( "${trans_file}.temp" );

    # reload file (could update in memory but it's a sanity check)
    #useless in non-module version#    config_load ();

}

# handle variables of type 'list'
# i.e. var=item1 (attr1=value1 attr2=value2 ...),item2,item3 (attr=value)...
sub
config_list_hash_add
{
    my $var_to_set = shift @_;

    # capitalize
    $var_to_set = "\u$var_to_set";

    #print STDERR "var_to_set = $var_to_set\n\t$transaction_vars{$var_to_set}\n\n";

    if ( ! defined ($transaction_vars{$var_to_set}) ) {
	print STDERR "This variable *$var_to_set* does not exist \n";
	exit(3);
    }

    # if variable is a static variable (not a proxy one)
    if ( $transaction_vars{$var_to_set} =~ /(get:|set:|trigger:)/ ) {
	print STDERR "You can't set (list add) a proxy variable \n";
	exit(3);
    }



    my @args = split (',', join (' ',@_));

    if( @_ == 1 ) { # if arg = 1, the list looks like item1,item2...
      print "OK => $_[ 0 ]\n";
      foreach( @args ) {
	my $args = $_;
	if ( $transaction_vars{ $var_to_set } ) {
	  my $equals = 0;
	  # Search if the element we want to add already exist in the list
	  foreach( split( ',', $transaction_vars{ $var_to_set } ) ) {
	    if( $_ eq $args ) {
	      $equals = 1;
	    }
	  }
	  # The element doesn't exist so we add it.
	  if( !$equals ) {
	    $transaction_vars{ $var_to_set } .= "," . $args;
	  }
	}
	# The current var is empty, we could add the element
	else {
	  $transaction_vars{ $var_to_set } .= $args;
	}
      }
      config_set ( $var_to_set, $transaction_vars{ $var_to_set } );
    }
    else {
      # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_set});
    my (@item, @val);

    foreach (@args) {
      my @tokens = split (' ',$_);

      for ( my $i = 0; $i < scalar(@tokens) ; $i+=2 ) {
	$item[ $i/2 ] = $tokens[ $i ];
	$val[ $i/2 ] = $tokens[ $i + 1 ];
      }


      # Check if values we want to add are not already in list
      my $all_equal;
      my $skip_list = 0;
      my $i;
      foreach ( keys %h ) {
	$all_equal = 1;
	for ( $i = 0; ( $i < @item && $all_equal ); $i++ ) {
	  # attr is'nt define or diffrent from the current ...
	  if ( ( !$h{ $_ }{ $item[ $i ] } || $h{ $_ }{ $item[ $i ] } ne $val[ $i ] ) &&
	       $all_equal ) {
	    $all_equal = 0;
	  }
	}
	if ( $all_equal ) {
	  $skip_list = 1;
	  last;
	}
      }

      if ( $skip_list ) {
	print STDERR "those value already exist in this list...\n";
	print STDERR "Skip to other arguments.\n";
	next;
      }

      my $item = find_item_name( %h );
      $h{$item} = {};

      # -- Add the element attributes and values --
      my %attrs = @tokens;
      while ( my ($attr, $value) = each %attrs ) {
	if ($attr) {
	  add_attr_value(\%h, $item, $attr, $value);
	}
      }
    }
    # -- Stringify the list and set the variable in the transaction file
    config_set ($var_to_set, stringify_hash (\%h) );
  }

}

# Example config-wrapper $$ -ls RulesList 1,result_result,ACCEPT 
# will replace the result_result attribute with the ACCEPT value
sub
config_list_set
{
    my $var_to_set = shift @_;
    my $separator = "";

    # capitalize
    $var_to_set = "\u$var_to_set";

    if ( ! defined ($transaction_vars{$var_to_set}) ) {
	print STDERR "This variable *$var_to_set* does not exist \n";
	exit (3);
    }

    # if variable is a static variable (not a proxy one)
    if ( $transaction_vars{$var_to_set} =~ /(get:|set:|trigger:)/ ) {
	print STDERR "You can't set (list add) a proxy variable \n";
	exit(3);
    }

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_set});

    my @args = split (',', join (' ',@_));
    my @ids_list;
    my @attr_values_list;
    
    # split the @args list in ids and arguments
    for (my $i=0; $i <= scalar(@args) -1; ++$i) {
	if ($args[$i] =~ m/^\d+$/) {
		push(@ids_list,$args[$i]);
	} else {
		push(@attr_values_list,$args[$i]);
	};
    };
   
    if( @_ == 1 ) { # if arg = 1, the list looks like item1,item2...
    	foreach my $key ( keys %h ) {
    	foreach my $id_to_modify (@ids_list) {
          if ($key == $id_to_modify) {
	  print "$key=?$id_to_modify\n";
	    foreach my $attr (keys %{$h{$key}}) {
	       my $value = $h{$key}{$attr};
	       for (my $attr_to_modify=0; $attr_to_modify <= scalar(@attr_values_list)-1; $attr_to_modify += 2) {
	          if ($attr_values_list[$attr_to_modify] =~ m/$attr/) {
		     print "(attr,value)=($attr,$value)\n";
		     $h{$key}{$attr} = $attr_values_list[$attr_to_modify+1];
		     print "(new_attr,new_value)=($attr,".$h{$key}{$attr}.")\n";
		   		}
        		}
        	}
	  }
	}
	}

    config_set ( $var_to_set, stringify_hash (\%h) );

    } else {
	    
      print "function part to be updated\n";
      foreach (@args) {

	my @tokens = split (' ',$_);

	my (@item, @val);
	for ( my $i = 0; $i < scalar(@tokens) ; $i+=2 ) {
	  $item[ $i/2 ] = $tokens[ $i ];
	  print "item is $tokens[ $i ]\n";
	  $val[ $i/2 ] = $tokens[ $i + 1 ];
	  print "value is $tokens[ $i +1 ]\n";
	}


	# Check if values we want to add are not already in list
	my $all_equal;
	my $kill_item;
	my $i;
	foreach ( keys %h ) {
	  $all_equal = 1;
	  for ( $i = 0; ( $i < @item && $all_equal ); $i++ ) {
	    if ( $h{ $_ }{ $item[ $i ] } ne $val[ $i ] &&
		 $all_equal ) {
	      $all_equal = 0;
	    }
	  }
	  print STDERR "will skip\n";
	  if ( $all_equal ) {
	    $kill_item = "$_";
	    delete $h{ $kill_item };
	    last;
	  }
	}

	#	print STDERR Dumper(\%h);
      }

      # -- Stringify the list and set the variable in the transaction file
      config_set ($var_to_set, stringify_hash (\%h) );
    }
}

# handle variables of type 'list'
# list multiple
# input required: var item1,attr1,value2,attr2,value2 item2,attr1,value2,attr2,value2
# 	i.e.: TCP_PUBLIC_SERVICES  ssh,forward,---,action,allow
#
# output desired: var=item1 (attr1=value1 attr2=value2 ...),item2,item3 (attr=value)...
# 	i.e.: TCP_PUBLIC_SERVICES=ssh (forward=--- action=allow)
sub
config_list_add
{
    my $var_to_set = shift @_;

    # capitalize
    $var_to_set = "\u$var_to_set";

    if ( ! defined ($transaction_vars{$var_to_set}) ) {
	print STDERR "This variable *$var_to_set* does not exist \n";
	exit(3);
    } 

    # if variable is a static variable (not a proxy one)
    if ( $transaction_vars{$var_to_set} =~ /(get:|set:)/ ) {
	print STDERR "You can't set (list add) a proxy variable \n";
	exit(3);
    }
    
    my $i = my $j = 0;

    my $joinlist = join (',',@_);
   
    $joinlist =~ s/\,\;/\,---\;/g;	    
    $joinlist =~ s/\,\,/\,---\,/g;	    
    
    #my @args = split (';', join (',',@_));
    my @args = split (';', $joinlist);

    for ($i=0; $i <= scalar(@args)-1; ++$i) {
	$args[$i] =~ s/\,/ /g;
    };

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_set});
    
    foreach (@args) {

	my @tokens = split (' ',$_);

	my $item = shift @tokens;
	if (! $item ) {
	    print STDERR "No element was given to add to the list \n";
	    exit(3);
	}
	
	# -- Add the given element --
	if ( ! $h{$item} ) {
	    $h{$item} = {};
	}

	# -- Add the element attributes and values --
	my %attrs = @tokens; 
	while ( my ($attr, $value) = each %attrs ) {
	    if ($attr) { add_attr_value(\%h, $item, $attr, $value); }
	}	

	#print STDERR Dumper(\%h);

    }

    # -- Stringify the list and set the variable in the transaction file
    config_set ($var_to_set, stringify_hash (\%h) );

}

# Example config-wrapper $$ -lains RulesList 1,result_result,ACCEPT,result_log_level,info,...
sub
config_list_addins
{
    my $var_to_set = shift @_;

    # capitalize
    $var_to_set = "\u$var_to_set";

    if ( ! defined ($transaction_vars{$var_to_set}) ) {
	print STDERR "This variable *$var_to_set* does not exist \n";
	exit(3);
    } 

    # if variable is a static variable (not a proxy one)
    if ( $transaction_vars{$var_to_set} =~ /(get:|set:)/ ) {
	print STDERR "You can't set (list add) a proxy variable \n";
	exit(3);
    }
    
    my $i = my $j = 0;

    my $joinlist = join (',',@_);
   
    $joinlist =~ s/\,\;/\,---\;/g;	    
    $joinlist =~ s/\,\,/\,---\,/g;	    
    
    #my @args = split (';', join (',',@_));
    my @args = split (';', $joinlist);

    for ($i=0; $i <= scalar(@args)-1; ++$i) {
	$args[$i] =~ s/\,/ /g;
    };

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_set});
    
    foreach (@args) {

	my @tokens = split (' ',$_);

	my $item = shift @tokens;
	if (! $item ) {
	    print STDERR "No element was given to add to the list \n";
	    exit(3);
	}
	
	#print STDERR Dumper(\%h);
	# -- Add the given element --
	if ( ! $h{$item} ) {
	    $h{$item} = {};
	} else {
		foreach my $key ( sort { "$a$b" =~ /^\d+$/ ? $b <=> $a : $a cmp $b } keys %h ) {
			if ($item =~ /^\d+$/ && ($key =~ /^\d+$/) && ($key >= $item)) { 
				$h{$key + 1} = $h{$key};
				delete $h{$key};
			};
		};
	};

	# -- Add the element attributes and values --
	my %attrs = @tokens; 
	while ( my ($attr, $value) = each %attrs ) {
	    if ($attr) { add_attr_value(\%h, $item, $attr, $value); }
	}	
	#print STDERR Dumper(\%h);

    }

    # -- Stringify the list and set the variable in the transaction file
    config_set ($var_to_set, stringify_hash (\%h) );

}

# Example config-wrapper $$ -lainc RulesList 1,result_result,ACCEPT,result_log_level,info,...
sub
config_list_addinc
{
    my $var_to_set = shift @_;

    # capitalize
    $var_to_set = "\u$var_to_set";

    if ( ! defined ($transaction_vars{$var_to_set}) ) {
	print STDERR "This variable *$var_to_set* does not exist \n";
	exit(3);
    } 

    # if variable is a static variable (not a proxy one)
    if ( $transaction_vars{$var_to_set} =~ /(get:|set:)/ ) {
	print STDERR "You can't set (list add) a proxy variable \n";
	exit(3);
    }
    
    my $i = my $j = 0;

    my $joinlist = join (',',@_);
   
    $joinlist =~ s/\,\;/\,---\;/g;	    
    $joinlist =~ s/\,\,/\,---\,/g;	    
    
    #my @args = split (';', join (',',@_));
    my @args = split (';', $joinlist);

    for ($i=0; $i <= scalar(@args)-1; ++$i) {
	$args[$i] =~ s/\,/ /g;
    };

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_set});
    
    foreach (@args) {

	my @tokens = split (' ',$_);

	my $item = shift @tokens;
	if (! $item ) {
	    print STDERR "No element was given to add to the list \n";
	    exit(3);
	}
	
	#print STDERR Dumper(\%h);
	# -- Add the given element --
	if ( ! $h{$item} ) {
	    $h{$item} = {};
	} else {
		foreach my $key ( sort { "$a$b" =~ /^\d+$/ ? $b <=> $a : $a cmp $b } keys %h ) {
    	if ($item =~ /^\d+$/ && ($key =~ /^\d+$/) && ($key >= $item)) {
      	$item = $key +1; 
				last;
			};
		};
	};

	# -- Add the element attributes and values --
	my %attrs = @tokens; 
	while ( my ($attr, $value) = each %attrs ) {
	    if ($attr) { add_attr_value(\%h, $item, $attr, $value); }
	}	
	#print STDERR Dumper(\%h);

    }

    # -- Stringify the list and set the variable in the transaction file
    config_set ($var_to_set, stringify_hash (\%h) );

}


# Example config-wrapper $$ -ldattr RulesList client,fw,server,net,port,www
sub
config_list_del_attr
{
    my $var_to_set = shift @_;
    my $separator = "";

    # capitalize
    $var_to_set = "\u$var_to_set";

    if ( ! defined ($transaction_vars{$var_to_set}) ) {
	print STDERR "This variable *$var_to_set* does not exist \n";
	exit (3);
    }

    # if variable is a static variable (not a proxy one)
    if ( $transaction_vars{$var_to_set} =~ /(get:|set:|trigger:)/ ) {
	print STDERR "You can't set (list add) a proxy variable \n";
	exit(3);
    }

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_set});

    my @args = split (',', join (' ',@_));
	for (my $i=0; $i <= scalar(@args) -1; $i=$i+1) {
		print "(argsi)=($args[$i])\n";
	}
    
	if( @_ == 1 ) { # if arg = 1, the list looks like item1,item2...
    	foreach my $key ( keys %h ) {
		my $kill_element = $key;
		my $to_delete =0;
			for (my $i=0; $i <= scalar(@args) -1; $i=$i+2) {
        		foreach my $attribute (keys %{$h{$key}}) {
		    		my $value = $h{$key}{$attribute};
		    		if (($args[$i] =~ m/$attribute/) && ( $args[$i+1] eq "$value")) {
							#print "(argsi,argsi+1)=($args[$i],$args[$i+1])\n";
							#print "(attribute,value)=($attribute,$value)\n";
						++$to_delete;
		   			}
				}
			}
			#print "$to_delete=".eval( scalar(@args)/2 )."\n";
			if ($to_delete == scalar(@args)/2) {
				delete $h{ $kill_element };
        	}
		}

    config_set ( $var_to_set, stringify_hash (\%h) );

    } else {
	    
      print "function part to be updated\n";
      foreach (@args) {

	my @tokens = split (' ',$_);

	my (@item, @val);
	for ( my $i = 0; $i < scalar(@tokens) ; $i+=2 ) {
	  $item[ $i/2 ] = $tokens[ $i ];
	  print "item is $tokens[ $i ]\n";
	  $val[ $i/2 ] = $tokens[ $i + 1 ];
	  print "value is $tokens[ $i +1 ]\n";
	}


	# Check if values we want to add are not already in list
	my $all_equal;
	my $kill_item;
	my $i;
	foreach ( keys %h ) {
	  $all_equal = 1;
	  for ( $i = 0; ( $i < @item && $all_equal ); $i++ ) {
	    if ( $h{ $_ }{ $item[ $i ] } ne $val[ $i ] &&
		 $all_equal ) {
	      $all_equal = 0;
	    }
	  }
	  print STDERR "will skip\n";
	  if ( $all_equal ) {
	    $kill_item = "$_";
	    delete $h{ $kill_item };
	    last;
	  }
	}

	#	print STDERR Dumper(\%h);
      }

      # -- Stringify the list and set the variable in the transaction file
      config_set ($var_to_set, stringify_hash (\%h) );
    }
}


sub
config_list_del
{
    my $var_to_set = shift @_;

    # capitalize
    $var_to_set = "\u$var_to_set";

    if ( ! defined ($transaction_vars{$var_to_set}) ) {
	print STDERR "This variable *$var_to_set* does not exist \n";
	exit (3);
    }

    # if variable is a static variable (not a proxy one)
    if ( $transaction_vars{$var_to_set} =~ /(get:|set:|trigger:)/ ) {
	print STDERR "You can't set (list add) a proxy variable \n";
	exit(3);
    }

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_set});

    my @args = split (',', join (' ',@_));

    if( @_ == 1 ) { # if arg = 1, the list looks like item1,item2...
		foreach my $key ( sort { "$a$b" =~ /^\d+$/ ? $a <=> $b : $a cmp $b } keys %h ) {
			my $kill_element = $key;
			foreach my $current_arg (@args) {
	  			if( $kill_element eq $current_arg ) {
	    			delete $h{ $kill_element };
		 		} elsif ($current_arg =~ /^\d+$/ && ($key =~ /^\d+$/) && ($key > $current_arg)) { 
					$h{$key - 1} = $h{$key};
					delete $h{$key};
				};
			}
      }
      config_set ( $var_to_set, stringify_hash (\%h) );
    }
    else {

      foreach (@args) {

	my @tokens = split (' ',$_);

	my (@item, @val);
	for ( my $i = 0; $i < scalar(@tokens) ; $i+=2 ) {
	  $item[ $i/2 ] = $tokens[ $i ];
	  $val[ $i/2 ] = $tokens[ $i + 1 ];
	}


	# Check if values we want to add are not already in list
	my $all_equal;
	my $kill_item;
	my $i;
	foreach ( keys %h ) {
	  $all_equal = 1;
	  for ( $i = 0; ( $i < @item && $all_equal ); $i++ ) {
	    if ( $h{ $_ }{ $item[ $i ] } ne $val[ $i ] &&
		 $all_equal ) {
	      $all_equal = 0;
	    }
	  }
	  print STDERR "will skip\n";
	  if ( $all_equal ) {
	    $kill_item = "$_";
	    delete $h{ $kill_item };
	    last;
	  }
	}

	#	print STDERR Dumper(\%h);
      }

      # -- Stringify the list and set the variable in the transaction file
      config_set ($var_to_set, stringify_hash (\%h) );
    }
}



sub
config_list_del_keys
{
   my $var_to_set = shift @_;

    # capitalize
    $var_to_set = "\u$var_to_set";

    if ( ! defined ($transaction_vars{$var_to_set}) ) {
    print STDERR "This variable *$var_to_set* does not exist \n";
    exit (3);
    }

    # if variable is a static variable (not a proxy one)
    if ( $transaction_vars{$var_to_set} =~ /(get:|set:)/ ) {
    print STDERR "You can't set (list add) a proxy variable \n";
    exit(3);
    }

    my @args = split (',', join (' ',@_));

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_set});


    foreach (@args) {

    my @tokens = split (' ',$_);

    my $item = shift @tokens;
    if (! $item ) {
        print STDERR "No element was given to delete from the list \n";
        exit(3);
    }

    # -- If the given element does not exist there's nothing to delete --
    if ( ! $h{$item} ) {
        print STDERR "This element *$item* does not appear in the list from variable *$var_to_set* \n";
        exit (3);
    }

    # -- If element attributes are given, delete only these attributes --
    if ( @tokens ) {
        foreach my $attr ( @tokens ) {
        if ($attr) { del_attr_value(\%h, $item, $attr); }
        }
    }

    # -- If no attributes are given, delete the whole element
    else {
        delete $h{$item};
    }

    #   print STDERR Dumper(\%h);
    }

    # -- Stringify the list and set the variable in the transaction file
    config_set ($var_to_set, stringify_hash (\%h) );
}

# Retrieve attributes and values of an item in a list
sub
config_list_get
{
    my $var_to_get = shift @_;

    # capitalize
    $var_to_get = "\u$var_to_get";

    if ( ! defined ($transaction_vars{$var_to_get}) ) {
	print STDERR "This variable *$var_to_get* does not exist \n";
	exit (3);
    } 

    my $item = shift @_;
    if ( !$item ) {
	print STDERR "No item was given in list_get \n";
	exit (3);
    }

    my $attr_to_get = shift @_;

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_get});

    # Print attributes and values for the item asked
    foreach my $item_key (keys %h) {

			# suivant si different de item_to_get
			if (($item_key =~ /^\d+$/) && ($item =~ /^\d+$/)) {
				next if ($item_key != $item);
			} else {
				next if ($item_key !~ $item);
			};
			if ( ref ($h{$item_key}) eq "HASH" ) {

					if ($attr_to_get) {
								print $h{$item_key}{$attr_to_get}."\n";
					} else {
						print $item_key." (";
						foreach my $attr_key ( keys (%{$h{$item_key}}) ) {
								# print "attr: ".$attr_key."\n";
								# print "val_attr:".$h{$item_key}{$attr_key}."\n";
							 print $attr_key."=".$h{$item_key}{$attr_key}." ";
						}
						print ")\n";
				}

	} else {
	    print STDERR "This item $item has no attributes set \n";
	    print $NULL_ITEM;
	}

    }

}

# build hash from comma-separated list
sub
build_hash_from_list
{
    my $list = shift @_;

    my %h;

    if ( $list !~ /\S+/ ) {
	return %h;
    }

    # split the List around character ","
    my @items = split (/,/,$list);

    foreach (@items) {

	my ($item, $attr_list) = /(\S+)\s*\((.*)\)/;
	# item      : a
	# attr_list : b=1 c=1 d=2

	if ($attr_list) {
	    my @vals = split ' ', $attr_list;

# problem with map with the compiled C version
#	    $h{$item} = { map { split '\s*=\s*'; } @vals };
# Doing this instead... :-(
	    foreach my $pair (@vals) {
		my ($attribute,$value) = split '\s*=\s*', $pair;
		$h{$item}{$attribute} = $value;
	    }

	} else {
	    $h{$_} = {}; 
	}
    }
    return %h;
}


# add an attribute/value to a list element
sub
add_attr_value {
    my ($h, $item, $attr, $val) = @_;

    print STDERR "Adding attribute $attr with value $val to item $item \n";

    $h->{$item}{$attr} = $val;
}

# del an attribute/value from a list element
sub
del_attr_value {
    my ($h, $item, $attr) = @_;

    print STDERR "Deleting attribute $attr from item $item \n";

    delete $h->{$item}{$attr};
}

sub find_item_name ()
{
  my %hash = @_;

  my @takin_item = ();
  foreach ( keys %hash ) {
    $takin_item[ $_ ] = 1;
  }

  #     for ( my $i = 0; $i < @takin_item; $i++ ) {
  # 	  if ( !$takin_item[ $i ] ) {
  # 	      return $i;
  # 	  }
  #     }

  # We don't want item to be 0
  for ( my $i = 1; $i < @takin_item; $i++ ) {
    if ( !$takin_item[ $i ] ) {
      return $i;
    }
  }

  # still No item 0
  if( !@takin_item ) {
    return 1;
  }
  else {
    return scalar( @takin_item );
  }
}

sub
stringify_hash
{
    my $hash_ref = shift @_;
    my $string     = "";
    my $attributes = "";
    my $separator  = "";

    foreach my $key ( sort { "$a$b" =~ /^\d+$/ ? $a <=> $b : $a cmp $b } keys %$hash_ref ) {
	$string .= $separator;
	$string .= "$key";

	# insert a white space
	foreach ( keys %{$hash_ref->{$key}} ) {
	    # handle attributes if any
	    if ( $hash_ref->{$key}{$_} ) {
		$attributes .= $_."=".$hash_ref->{$key}{$_}." ";
	    }
	}
	if ( $attributes && $attributes =~ /\S+/) {
	    $attributes =~ s/\s*$//;
	    $string .= " (".$attributes.")";
	    $attributes = "";
	}
	$separator = ",";
    }

    return $string;
}

# Get one all variables and their content
# Here values are printed to stdout
sub
config_get_all
{
    foreach (sort keys %transaction_vars) {
	print "$_ = $transaction_vars{$_}\n";
    }
}


# arg1       : variable name
# args 2,... : optional arguments
# e.g. config-wrapper transaction -g UserShell some_user_name
# Here values are printed to stdout
sub
config_get
{
    my $variable_name = shift @_;

    if (! defined ($variable_name) ) {
	print STDERR "No argument given in config_get\n";
	print "???\n";
	return;
    }

    # capitalize
    $variable_name = "\u$variable_name";

    if ( defined ($transaction_vars{$variable_name})) { 
	if ( $transaction_vars{$variable_name} =~ /\S+/ ) {   # if not empty

	    # if variable is a static variable, print it to stdout
	  if ( $transaction_vars{$variable_name} !~ /(get:|set:|trigger:)/ ) {
	    print "$transaction_vars{$variable_name}\n";
	  }
	    # else perform appropriate action
	    else {
		perform_action ("get", $transaction_vars{$variable_name}, @_);
	    }
	  } else {
	      print STDERR "$variable_name : empty variable \n";
	      print $NULL_ITEM
#	    exit (1);
	  }
    } else {
	  print STDERR "!!! Parameter $variable_name does not exist !!!\n";
	  print $NULL_VAR_NAME;
#	exit (2);
     }
}

# Prints a comma-separated list of values for all variable names passed as arguments.
# Here we don't provide proxy variables functionnalities, we just print the contents.
# Usage example: config-wrapper transation --mget Eth0IP Eth0Mask Eth0Mac Eth0Driver
sub
config_multiple_get
{
    my @variable_names = @_;

    my $output    = "";
    my $separator = "";
    foreach (@variable_names) {
	# capitalize
	$_ = "\u$_";

	if ( defined ($transaction_vars{$_})) { 
	    $output .= "$separator$transaction_vars{$_}"; # even if empty
	} else {
	    print STDERR "!!! Parameter $_ does not exist !!!\n";
	    exit (2);
	}
	$separator = ";"; # old: $separator = ",";
    }
    print "$output\n";
}

# Prints a comma-separated list of values for all the attributes of the same kind of a
# composed variable.
# Here we don't provide proxy variables functionnalities, we just print the contents.
# Usage example: config-wrapper transation --listmget RulesList server
sub
config_list_multiple_get
{
    my $var_to_get = shift @_;
    my $output    = "";
    my $separator = "";

    # capitalize
    $var_to_get = "\u$var_to_get";

    if ( ! defined ($transaction_vars{$var_to_get}) ) {
	print STDERR "This variable *$var_to_get* does not exist \n";
	exit (3);
    } 

    my $attr_to_get = shift @_;
    if ( !$attr_to_get ) {
	print STDERR "No attribute was given in list_multiple_get \n";
	exit (3);
    }

    # -- Get variable content --
    my %h = build_hash_from_list ($transaction_vars{$var_to_get});

    # Print attributes and values for the item asked
    foreach my $item_key (keys %h) {

	if ( ref ($h{$item_key}) eq "HASH" ) {

		if ($attr_to_get) {
	  		$output .= "$separator$h{$item_key}{$attr_to_get}"; # even if empty
		} else {
			foreach my $attr_key ( keys (%{$h{$item_key}}) ) {
				# print "attr: ".$attr_key."\n";
				# print "val_attr:".$h{$item_key}{$attr_key}."\n";
				print $attr_key."\n";
				print $h{$item_key}{$attr_key}."\n";
			}
		}

	} else {
	    print STDERR "This item $attr_to_get has no attributes set \n";
	    print $NULL_ITEM;
	}
	$separator = ","; # old: $separator = ",";

    }
    print "$output\n";
}

# Create a hash table from differences between the transaction
# working configuration file and the master config file
sub
config_diff
{
    my %diff = compute_diff ($config_filename, "${trans_file}");
    return %diff;
}

# arg1 (string): action type (get or set)
# arg2 (string): content of the virtual/proxy var 
#                (i.e. get and/or set method on this variable)
# @arg3 (list) : parameters to pass to the action
#
sub
perform_action
{
    my $action_type = shift @_;
    my $content     = shift @_;

    $content =~ s/^\s+//;		# no leading white
    $content =~ s/\s+$//;		# no trailing white


    # Take args into account when splitting
    # (for set: action, where args are appended to the 'virtual/proxy var' content 
    #  during the transaction)
    my ($methods, $args) = ($content =~ /\'(.*)\'\s*(.*)/);
    # exit if this variable can't perform the requested action (i.e. this method don't apply to this var)


    if ( $methods !~ /$action_type/ ) { exit (1) }; # 'variable exists but is empty', we might need a special error# here

    # For get: action, we can have parameters (which are not appended 
    # to the virtual/proxy var content since the action is performed immediately,
    # i.e. we don't wait for a commit)
    unless ( $args ) {
	$args = "";
	foreach (@_) {
	    $args .= "\"$_\" ";
	}
#	$args = join (" ", @_);  # Does not seem to escape quotes (so an arg containing a space becomes 2 args after the join...)
    }

    # Split in case there are both a get,set and trigger actions possible
    my ($action1, $action2, $action3) = split ( ';',  $methods, 3);

    my $action;
    # Isolate the get:, set: or trigger: in action(1|2|3)
    foreach ( "1", "2", "3" ) {
      my $current_action = eval( '$' . "action" . $_ );
      if( $current_action && ($current_action =~ /${action_type}:/) ) {
	# retrieve the action only.
	# $action =~ s/^\s*$action_type:\s*(.*)[\s\']*$/$1/;
	$current_action =~ s/${action_type}://;
	$action = $current_action;
      }
    }

    if ( !$action ) {
      print STDERR "The action you want to execute is'nt set: get: or trigger:\n";
      exit( 1 ); # We must find an correct exit value here.
    }
    # separate script name from script flags
    my $flags;
    ($action, $flags) = ( $action =~ /(\S+)(.*)/ );

    # Execute with the remaining arguments
    print STDERR "$action_type Action to perform: $action $transaction_name $flags $args \n";
#    system ("sudo", "$scripts_default_dir/$action", $flags, $args);
    # Keep the command line in one string to allow redirection to /dev/null for instance

    system ("PATH=\$PATH:/sbin:/usr/sbin sudo $scripts_default_dir/$action $transaction_name $flags $args");
}



#-------------------------------------------------------------
# Backup and Restore Functions
#-------------------------------------------------------------
# config_backup
# config_restore
# config_load
# config_apply_all
#

# Load the master configuration file into a Hash
sub
config_load
{
  print STDERR "load the file $config_filename\n";
    if (scalar (%master_vars)) { %master_vars = (); }

    %master_vars = file_load ("${config_filename}");

    return;
}

# Args : $1 optional destination filename with path
sub
config_backup
{
    my $dest_filename = shift @_;

    defined ($dest_filename) or $dest_filename = "${config_filename}.backup";

    # backup configuration file
    [ -f "$dest_filename" ] && unlink ("$dest_filename");
    system ("cp", $config_filename, $dest_filename);

    print "Configuration backed up as $dest_filename \n";
}


# Args : $1 optional configuration backup filename with path
sub
config_restore
{
    my $config_backup = shift @_;

    defined ($config_backup) or $config_backup = "${config_filename}.backup";

    # restore configuration file, then call event handler with all variables
    # TODO : see if we need a differential restore (apply only changes between
    #        backup file and current file if present
    [ -f "$config_backup" ] or do { 
	print "$config_backup file was not found, restoration nor performed\n";
	return;
    };
    [ -f "$config_filename" ] && unlink ("$config_filename");
    system ("cp", $config_backup, $config_filename);

    print "$config_filename restored, restoring system now...\n";
    config_apply_all ();
}


sub
config_apply_all
{
    # load master configuration file
    config_load ();

    # trigger all events at once
    print "*** TRIGGER ALL KNOWN EVENTS ***\n";
    system ("sudo", "/usr/bin/event-handler.pl", $transaction_name, keys %master_vars);
}


#-------------------------------------------------------------
# Backup and Restore Functions
#-------------------------------------------------------------
#
# Utility functions
#

# Load a configuration file into a Hash
# Parameters: filename
# Return: a hash table


sub
file_load
{
    my $filename = shift @_;
    my %hash = ();

#    print STDERR "Loading config file: $filename \n";

#-- If you don't want to use tie, uncomment these lines --
    # parse configuration file and store variables and values
# WARNING THIS METHODE DON'T RESTORE CORRECTLY LIST !!!
#     open (CONFIG, "$filename") or die "Can't open config file: $!";
#     while (<CONFIG>) {
# 	   chomp;                  # no newline
# 	   s/#.*//;                # no comments
# 	   s/^\s+//;               # no leading white
# 	   s/\s+$//;               # no trailing white
# 	   next unless length;     # anything left?
# 	   my ($var, $value) = split(/\s*=\s*/, $_);
# 	   $hash{$var} = $value;
#     }
#     close (CONFIG);

#-- If you don't want to use tie, comment out this line --
    tie %hash, 'backend::config', $filename;

    return %hash;
}


# Create a hash table from differences between the 
# source file and the dest file (for same parameter names)
# Parameters: filename1, filename2
sub
compute_diff
{
  my $filename1 = shift @_;
  my $filename2 = shift @_;

  my %hash1 = ();
  my %hash2 = ();
  my %diff  = ();

  %hash1 = file_load ($filename1);
  %hash2 = file_load ($filename2);

  # compute diffs
  foreach (keys %hash1) {
    if ( defined ($hash2{$_}) ) { 
      if ( $hash2{$_} ne $hash1{$_} ) {
	$diff{$_} = $hash2{$_};
	print "$_ parameter had value: $hash1{$_}\n";
	#		print "$_ parameter has new value: $hash2{$_}\n";
      }
    }
  }
  return %diff;
}
