#!/usr/bin/perl -w
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2000,2001 Mandrakesoft
# Author Renaud Chaillat <rchaillat@mandrakesoft.com>
#
# $Id: event-handler.pl,v 1.2 2002/12/04 15:26:16 philippe Exp $
#

# version 0.2 (with transaction name)

# Usage: event-handler transaction_name event1 [event2] ...
# The transaction_name is pased to scripts 
# (it's not used anywhere else in the event-handler)

use strict;
# to process templates
use lib qw(/usr/lib/naat/);
use backend::config;
use backend::util; 

#----------------------------------------
# Read configuration file 
# /etc/naat/backend.conf
# and set default values
#----------------------------------------

# parse backend configuration file and store variables and values
my %backend_conf = (); 
#tie %backend_conf, 'backend::config', '/etc/naat/backend.conf';

open (CONFIG, "/etc/naat/backend.conf") or die "Can't open config file: $!";
while (<CONFIG>) {
    chomp;                  # no newline
    s/#.*//;                # no comments
    s/^\s+//;               # no leading white
    s/\s+$//;               # no trailing white
    next unless length;     # anything left?
    my ($var, $value) = split(/\s*=\s*/, $_, 2);
    $value =~ s/\"//g;      # remove "s
    $backend_conf{$var} = $value;
}
close (CONFIG);

# prefix to add to the path of files created from templates 
# (without final slash) 
my $dest_prefix = "";
if ( defined ($backend_conf{"DEST_PREFIX"}) ) { 
    $dest_prefix = $backend_conf{"DEST_PREFIX"};   
} else {
    print STDERR "Destination set to default: /var/tmp\n";
    $dest_prefix = "/var/tmp";       
}
print STDERR "Dest prefix: $dest_prefix\n";

# configuration filename
# (database for system parameters)
my $config_filename = "";
if ( defined ($backend_conf{"CONFIGURATION_FILE"}) ) { 
    $config_filename = $backend_conf{"CONFIGURATION_FILE"};   
} else {
    print STDERR "Default configuration file used: /var/lib/naat/configuration\n";
    $config_filename = "/var/lib/naat/configuration";       
}
print STDERR "Configuration file: $config_filename\n";

# events path			
my $events_path = "";
if ( defined ($backend_conf{"EVENTS_PATH"}) ) { 
    $events_path = $backend_conf{"EVENTS_PATH"};   
} else {
    print STDERR "Events path set to default: /usr/share/naat/events\n";
    $events_path = "/usr/share/naat/events";       
}
print STDERR "Events path: $events_path\n";

# default template dir
my $templates_default_dir = "";
if ( defined ($backend_conf{"TEMPLATES_DEFAULT_DIR"}) ) { 
    $templates_default_dir = $backend_conf{"TEMPLATES_DEFAULT_DIR"};   
} else {
    print STDERR "Template default dir set to default: /usr/share/naat/templates\n";
    $templates_default_dir = "/usr/share/naat/templates";       
}
print STDERR "Templates dir: $templates_default_dir\n";

# default scripts dir
my $scripts_default_dir = "";
if ( defined ($backend_conf{"SCRIPTS_DEFAULT_DIR"}) ) { 
    $scripts_default_dir = $backend_conf{"SCRIPTS_DEFAULT_DIR"};   
} else {
    print STDERR "Scripts default dir set to default: /usr/share/naat/scripts\n";
    $scripts_default_dir = "/usr/share/naat/scripts";       
}
print STDERR "Scripts dir: $scripts_default_dir\n";


#----------------------------------------
# Tie our hash table to the config file
#----------------------------------------

my %conf;
tie %conf, 'backend::config', $config_filename;



#----------------------------------------
# Process arguments
#----------------------------------------
unless (scalar (@ARGV) >= 2) {
    print STDERR "\nNo argument received by event-handler\n";
    print STDERR "Usage : event-handler transaction_name event1 [event2] ...\n"; 
    exit 0;
}

# Retrieve the transaction ID and the events list
my $transaction_name = shift @ARGV;
my @events = @ARGV;

#DEBUG
foreach (@events) { print STDERR "Event : $_ \n"; }


#----------------------------------------
# Perform the actual Work
#----------------------------------------
my %actions = ();
(my $runlevel = `/sbin/runlevel`) =~ s/. (\d)/$1/  ;
chomp $runlevel;
print STDERR "RUNLEVEL: $runlevel\n";

print STDERR "\n1. PRE:\n";
%actions = unique_actions("pre", @events);
#foreach (keys %actions) {
#    print STDERR "$_:\n @{$actions{$_}}\n";
#} 
handle_templates (@{$actions{"templates"}});
handle_scripts   (@{$actions{"scripts"}});
handle_services  ("stop",    @{$actions{"stop"}});
handle_services  ("reload",  @{$actions{"reload"}});
handle_services  ("restart", @{$actions{"restart"}});
handle_services  ("start",   @{$actions{"start"}});

print STDERR "\n2. MAIN:\n";
%actions = unique_actions("main", @events);
handle_templates (@{$actions{"templates"}});
handle_scripts   (@{$actions{"scripts"}});
handle_services  ("stop",    @{$actions{"stop"}});
handle_services  ("reload",  @{$actions{"reload"}});
handle_services  ("restart", @{$actions{"restart"}});
handle_services  ("start",   @{$actions{"start"}});

print STDERR "\n3. POST:\n";
%actions = unique_actions("post", @events);
handle_templates (@{$actions{"templates"}});
handle_scripts   (@{$actions{"scripts"}});
handle_services  ("stop",    @{$actions{"stop"}});
handle_services  ("reload",  @{$actions{"reload"}});
handle_services  ("restart", @{$actions{"restart"}});
handle_services  ("start",   @{$actions{"start"}});



#------------------------------------------
# Utility functions   
#------------------------------------------
# handle_services
# handle_templates
# handle_scripts
# unique_actions
#

# Start or Stop services in current runlevel
# *only* if these services are listed as S##service_name
# (i.e. don't handle services marked as killed in this runlevel)
# args : $1 : start or stop (just a hint)
#        @2 : list of services
sub 
handle_services
{
    my $action = shift @_;
    if ( (@_) == 0 ) { return; }  # if no services listed

    # construct list of services string for the shell
    my $services;
    if ( (@_) == 1 ) { $services = shift @_; }
    if ( @_ ) { 
	$services = "{".shift @_;
	foreach (@_) {
	    $services = "$services,$_";
	}
	$services = $services."}";
    }
    print STDERR "=====> Check services to $action: $services\n";

    # list services in correct order
    my @real_services;
    if ( $action eq "stop" ) {   # reverse order to stop services
	@real_services = `ls -r /etc/rc${runlevel}\.d/S[0-9][0-9]$services 2>/dev/null`; 
    } else  {
	@real_services = `ls /etc/rc${runlevel}\.d/S[0-9][0-9]$services 2>/dev/null`; 
    } 
    print STDERR "* C. Real order of services to $action: \n @real_services\n";

    # Starting or stopping services accordingly
    foreach (@real_services) {
	s/^.*S[0-9]*/ /;  # get the service name only
	chomp;            # no trailing newline
	print STDERR "service $_ $action\n";
	system "/sbin/service $_ $action"; 
    }
    
}

# args : @list of templates (with path $templates_default_dir/<template-name>)
sub 
handle_templates
{
    # processTemplate ($confref, $template_filename, $sourcedir, $dest_filename)

    my ($sourcedir, $template_filename);
    foreach (@_) {
	if ( m:^$templates_default_dir/.*: ) {
	    ($sourcedir, $template_filename) = m:(^$templates_default_dir)/(.*):;

	    print STDERR "* A. Template : $template_filename\n";
	    print STDERR "     --------> to search in dir :  $sourcedir\n";
	    
	    if ( -f "${sourcedir}/${template_filename}" ||
		 -d "${sourcedir}/${template_filename}" ) {
		print STDERR "   ! Applying template on ${dest_prefix}/${template_filename}...\n\n";
	        backend::util::processTemplate (\%conf, 
						"/$template_filename",
						"$sourcedir",
						"${dest_prefix}/${template_filename}");
	    } else {
		    print STDERR " ? Template not found.\n\n";
	    }
	} else {
	    print STDERR "Wrong template path : $_\n";
	}

    }
}

# args : @list of scripts (with the full path, e.g. /usr/share/naat/scripts/script_name)
#
# *** The transaction name is given as the first and only parameter to the scripts ***
#
# Note: scripts launched because of an event can't have arguments as of now 
# Event hints in the events folders /usr/share/naat/events only contain the script 
#  name, e.g. script--check-services.pl.
# Only scripts executed because of a proxy variable can have arguments in addition to 
# the transaction name.
sub 
handle_scripts
{
    foreach (@_) {
	print STDERR "* B. Execute script : $_\n";
	system ("PATH=\$PATH:/sbin:/usr/sbin sudo $_ $transaction_name");
    }
}


# args : $1 : "pre", "main", or "post"
#        @2 : events list  ("local-ip", "local-mask", ...)
# Returns : hash table of actions lists
sub 
unique_actions 
{
    my $action_level = shift @_;  # pre, main or post level 
    my @hints_list;               # filenames to tell what action should be performed
    my $events;                   # string to construct {event1,event2,...} for the shell
    my @scripts;
    my @templates;
    my @start_services;
    my @stop_services;
    my @reload_services;
    my @restart_services;
    my @unique_scripts;
    my @unique_templates;
    my @unique_start_services;
    my @unique_stop_services;
    my @unique_reload_services;
    my @unique_restart_services;

    # the hash table which will be returned 
    my %unique_actions = ();

    # construct the events list
    if ((@_) == 0) { $events = "*"; }
    if ((@_) == 1) { $events = shift @_; }
    if (@_) { 
	$events = "{".shift @_;
	foreach (@_) {
	    $events = "$events,$_";
	}
	$events = $events."}";
    }
    print STDERR "==> $events\n";

    # find all actions to perform for this level (pre, main or post)
    @hints_list = `ls $events_path/$events/$action_level/* 2>/dev/null`;
#    print STDERR "ls $events_path/$events/$action_level/* 2>/dev/null \n";

    # dispatch action types and prepare sorting
    for (@hints_list) {
	chomp;	# no newline
	/template--/ and do { 
# to use if we want templates in modules folders
#	  s:(.*)/events/.*/template-(.*):$1/templates/$2:;
	  s:.*/template--(.*):$templates_default_dir/$1:; # prepend real path to hint
	  s:--:/:g; # change -- into / to create a path from the hint
#	  s:_:-:g; # change _ into - to obtain the real filenames
	  @templates = (@templates, $_);
	  next;
	};
	/script--/   and do { 
# to use if we want templates in modules folders
#	  s:(.*)/events/.*/script-(.*):$1/scripts/$2:;      
	  s:.*/script--(.*):$scripts_default_dir/$1:;     # prepend real path to hint
	  s:--:/:g; # change -- into / to create a path from the hint
#	  s:_:-:g; # change _ into - to obtain the real filenames
	  @scripts = (@scripts, $_);
	  next;
	};
	/reload--/  and do { 
	  s:.*/reload--(.*):$1:;                           # remember only service name to restart
	  s:--:/:g; # change -- into / to create a path from the hint
#	  s:_:-:g; # change _ into - to obtain the real filenames
	  @reload_services = (@reload_services, $_);
	  next;
	};
	/restart--/  and do { 
	  s:.*/restart--(.*):$1:;                           # remember only service name to restart
	  s:--:/:g; # change -- into / to create a path from the hint
#	  s:_:-:g; # change _ into - to obtain the real filenames
	  @restart_services = (@restart_services, $_);
	  next;
	};
        /\/start--/  and do { 
	  s:.*/start--(.*):$1:;                             # remember only service name to start
	  s:--:/:g; # change -- into / to create a path from the hint
#	  s:_:-:g; # change _ into - to obtain the real filenames
	  @start_services = (@start_services, $_);
	  next;
	};
	/stop--/     and do { 
	  s:.*/stop--(.*):$1:;                              # remember only service name to stop
	  s:--:/:g; # change - into / to create a path from the hint
#	  s:_:-:g; # change _ into - to obtain the real filenames
	  @stop_services = (@stop_services, $_);
	  next;
	};
    }

    # sort unique templates, scripts and services
    my %seen = ();
    @unique_scripts = grep { ! $seen{$_} ++ } @scripts;
    %seen = ();
    @unique_templates = grep { ! $seen{$_} ++ } @templates;
    %seen = ();
    @unique_start_services = grep { ! $seen{$_} ++ } @start_services;
    %seen = ();
    @unique_stop_services = grep { ! $seen{$_} ++ } @stop_services;
    %seen = ();
    @unique_reload_services = grep { ! $seen{$_} ++ } @reload_services;
    %seen = ();
    @unique_restart_services = grep { ! $seen{$_} ++ } @restart_services;

    if ( scalar (@unique_templates) > 0 ) {
	push (@{$unique_actions{"templates"}}, @unique_templates);
    }
    if ( scalar (@unique_scripts) > 0 ) {
	push (@{$unique_actions{"scripts"}}, @unique_scripts);
    }
    if ( scalar (@unique_start_services) > 0 ) {
	push (@{$unique_actions{"start"}}, @unique_start_services);
    }
    if ( scalar (@unique_stop_services) > 0 ) {
	push (@{$unique_actions{"stop"}}, @unique_stop_services);
    }
    if ( scalar (@unique_reload_services) > 0 ) {
	push (@{$unique_actions{"reload"}}, @unique_reload_services);
    }
    if ( scalar (@unique_restart_services) > 0 ) {
	push (@{$unique_actions{"restart"}}, @unique_restart_services);
    }

    foreach (keys %unique_actions) {
	print STDERR "Unique $_:\n ". join ("\n ", @{$unique_actions{$_}})."\n";
    } 
    
    return %unique_actions;
}




