#!/usr/bin/perl -wT
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

#----------------------------------------------------------------------
# copyright (C) 1999, 2000 e-smith, inc.
# copyright (C) 2001 Mandrakesoft
# 		
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 		
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 		
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
# 
# Technical support for this program is available from e-smith, inc.
# Please visit our web site www.e-smith.net for details.
#----------------------------------------------------------------------

package backend::util;

use lib qw(/usr/lib/naat/);
use backend::config;

 use Data::Dumper;

use strict;
use Text::Template 'fill_in_file';
use POSIX 'setsid';
use File::Basename;
use File::Path;
#use Errno;

#------------------------------------------------------------
# 
#------------------------------------------------------------

BEGIN
{
}

#------------------------------------------------------------
# subroutine to set real UID to effective UID and real GID
# to effective GID
#------------------------------------------------------------

sub setRealToEffective ()
{
    $< = $>;
    $( = $);
}

#------------------------------------------------------------
# The processTemplate function takes as arguments a tied reference to
# the configuration file, and the name of the output file to
# produce. The template is found by prepending /etc/e-smith/templates
# to the filename. The mode of the output file is copied from the mode
# of the template file.
#
# We also allow a parallel template hierarchy ${source}-custom 
# which override the standard templates in ${source}.
#
# The templates in ${source}-custom are _not_ merged with the standard
# templates in ${source} - the ones in ${source}-custom are used 
# in preference. Directory templates must be populated, either
# with fragments or symlinks to the real templates
#
# It is possible to override a file based template with a customised
# directory based template (and vice-versa if you really want to).
#------------------------------------------------------------

sub processTemplate
{
    my ($confref, $template_filename, $sourcedir, $dest_filename) = @_;
    my @templates = ();

    defined $sourcedir or $sourcedir = "/usr/share/naat/templates";
    defined $dest_filename or $dest_filename = $template_filename;

    # test if destdir exists and create it if needed
    my $destdir = dirname ($dest_filename);
    # print "Util: Destination path: $destdir\n";
    unless ( -d $destdir ) {
	print "Folder does not exists, creating...\n";
	mkpath ($destdir, 1);
    }
    print "\n";

    -r "$sourcedir$template_filename"
	|| die "Cannot read template $sourcedir$template_filename: $!\n";

    open (TEMPLATEWR, ">$dest_filename")
	|| die "Cannot open output file $dest_filename: $!\n";

    #------------------------------------------------------------
    # Gordon Rowell <Gordon.Rowell@e-smith.com.au> April 29, 2000
    # Use custom template for this file, if custom template exists
    #------------------------------------------------------------
    my $customSource = "${sourcedir}-custom";

    if ( -r "${customSource}${template_filename}" )
    {
	warn "Customised templates used from ${customSource}${template_filename}\n";
	$sourcedir = $customSource;
    }

    if (-d "$sourcedir$template_filename")
    {
		#-------------------------------------------------- 
		# The template is to be put together from fragments
		# List the template fragments to put together. Make sure
		# to drop the "."and ".." entries.
		#-------------------------------------------------- 

		opendir (DIR, "$sourcedir$template_filename")
			|| die "Can't open directory $sourcedir$template_filename\n";

		#-------------------------------------------------- 
		# Modification by JMorrison, January 18, 2000:
		# Set up template pieces so that "template-begin" is
		# guaranteed to be the first piece, followed by the other
		# pieces in alphabetical order, ending with "template-end".
		#-------------------------------------------------- 

		@templates = grep (!/^(\.\.?)$/, readdir (DIR));
		@templates = grep (!/^template-begin$/, @templates);
		@templates = grep (!/^template-end$/, @templates);
		@templates = sort (@templates);
		-f "$sourcedir$template_filename/template-begin" and
		    unshift @templates, 'template-begin';
		-f "$sourcedir$template_filename/template-end" and
		    push @templates, 'template-end';

		# Probably need to untaint data here

		closedir (DIR);

		$sourcedir .= "$template_filename/";

    }
    elsif (-f "$sourcedir$template_filename")
    {
    	unshift(@templates, $template_filename);
    }
    else
    {
        die "Don't know what to do with template $sourcedir$template_filename: $!\n";
    }
       
    my $file;
    foreach $file (@templates)
    {
        #-------------------------------------------------- 
        # Gordon Rowell <Gordon.Rowell@e-smith.com.au> April 29, 2000
        # We only want to process regular files - skipping RCS directories
        # and any other random things we might encounter. A symlink
        # to a regular file will do too.
	# This should probably be tightened to require template
	# names to match a regexp
        #-------------------------------------------------- 
        unless ( -f "${sourcedir}${file}" ||
	    (-l "${sourcedir}${file}" && -f readlink "${sourcedir}${file}" ))
        {
	    warn "Ignoring template ${sourcedir}${file} - not a regular file\n";
	    next;
	}
	# Arrange for useful warning messages
	local $SIG{__WARN__} =
	    sub { print STDERR "In ${sourcedir}${file}: $_[0]"; };
	fill_in_file ("$sourcedir$file",
		      HASH    => $confref,
#???		      PACKAGE => 'esmith::parameters',
		      OUTPUT  => \*TEMPLATEWR)
	    || die "Cannot process template $sourcedir$file: $!\n";
    }

    close TEMPLATEWR;
    # We aren't interested in any of the old copies or proposed new
	# copies that RPM leaves lying around. Tidy up - remove them.
    
    -f "$template_filename.rpmsave" and unlink "$template_filename.rpmsave";
    -f "$template_filename.rpmnew" and unlink "$template_filename.rpmnew";
    -f "$template_filename.rpmorig" and unlink "$template_filename.rpmorig";
    
    chown 0, 0, $template_filename;
}

#------------------------------------------------------------
# 
#------------------------------------------------------------

sub chownFile ($$$)
{
    my ($user, $group, $file) = @_;

    my ($discard1, $discard2, $uid) = getpwnam ($user);
    my ($discard3, $discard4, $gid) = getgrnam ($group);

    chown ($uid, $gid, $file);
}

#------------------------------------------------------------
# Convert IP address from "xxx.xxx.xxx.xxx" notation to a 32-bit
# integer.
#------------------------------------------------------------

sub IPquadToAddr ($)
{
    my ($quad) = @_;
    if ($quad =~  /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/)
    {
        return ($1 << 24) + ($2 << 16) + ($3 << 8) + $4;
    }
    return 0;
}

#------------------------------------------------------------
# Convert IP address from a 32-bit integer to "xxx.xxx.xxx.xxx"
# notation.
#------------------------------------------------------------

sub IPaddrToQuad ($)
{
    my ($addrBits) = @_;
    return sprintf ("%d.%d.%d.%d", ($addrBits >> 24) & 0xff,
		                   ($addrBits >> 16) & 0xff,
		                   ($addrBits >> 8) & 0xff,
		                    $addrBits & 0xff);
}

#------------------------------------------------------------
# Convert IP address from a 32-bit integer to reversed
# "xxx.xxx.xxx.xxx.in-addr.arpa" notation for BIND files.
#------------------------------------------------------------

sub IPaddrToBackwardQuad ($)
{
    my ($addrBits) = @_;
    return sprintf ("%d.%d.%d.%d.in-addr.arpa.", $addrBits & 0xff,
		                                 ($addrBits >> 8) & 0xff,
                                                 ($addrBits >> 16) & 0xff,
                                                 ($addrBits >> 24) & 0xff);
}

#------------------------------------------------------------
# Given an IP address and netmask (both in "xxx.xxx.xxx.xxx" format)
# compute the network and broadcast addresses and output them in the
# same format.
#------------------------------------------------------------

sub computeNetworkAndBroadcast ($$)
{
    my ($ipaddr, $netmask) = @_;

    my $ipaddrBits  = IPquadToAddr ($ipaddr);
    my $netmaskBits = IPquadToAddr ($netmask);

    my $network   = IPaddrToQuad ($ipaddrBits & $netmaskBits);
    my $broadcast = IPaddrToQuad ($ipaddrBits | (~ $netmaskBits));

    return ($network, $broadcast); 
}

#------------------------------------------------------------
# Given an IP address and netmask, the computeLocalNetworkPrefix
# function computes the network prefix for local machines.
# 
# i.e. for an IP address of 192.168.8.4 and netmask of 255.255.255.0,
# this function will return "192.168.8.".
# 
# This string is suitable for use in configuration files (such as
# /etc/proftpd.conf) when the more precise notation
# 
#     xxx.xxx.xxx.xxx/yyy.yyy.yyy.yyy
# 
# is not supported.
#------------------------------------------------------------

sub computeLocalNetworkPrefix ($$)
{
    my ($ipaddr, $netmask) = @_;

    my $ipaddrBits  = IPquadToAddr ($ipaddr);
    my $netmaskBits = IPquadToAddr ($netmask);

    # check for class A IP address
    if (($netmaskBits & 0xffffff) == 0)
    {
	return sprintf ("%d.", ($ipaddrBits >> 24) & 0xff);
    }

    # check for class B IP address
    if (($netmaskBits & 0xffff) == 0)
    {
	return sprintf ("%d.%d.", ($ipaddrBits >> 24) & 0xff,
		                  ($ipaddrBits >> 16) & 0xff);
    }

    # check for class C IP address
    if (($netmaskBits & 0xff) == 0)
    {
	return sprintf ("%d.%d.%d.", ($ipaddrBits >> 24) & 0xff,
		                     ($ipaddrBits >> 16) & 0xff,
		                     ($ipaddrBits >> 8) & 0xff);
    }

    # Bummer. This subnet cannot be described in prefix notation so
    # we'll have to return the entire IP address.

    return $ipaddr;
}

#------------------------------------------------------------
# Given an IP address and netmask, the computeLocalNetworkShortSpec
# function computes a valid xxx.xxx.xxx.xxx/yyy specifier where yyy
# is the number of bits specifying the network.
# 
# i.e. for an IP address of 192.168.8.4 and netmask of 255.255.255.0,
# this function will return "192.168.8.0/24".
# 
# This string is suitable for use in configuration files (such as
# /etc/proftpd.conf) when the more precise notation
# 
#     xxx.xxx.xxx.xxx/yyy.yyy.yyy.yyy
# 
# is not supported.
# 
# NOTE: This code only handles standard class A, B or C networks.
#------------------------------------------------------------

sub computeLocalNetworkShortSpec ($$)
{
    my ($ipaddr, $netmask) = @_;

    my $ipaddrBits  = IPquadToAddr ($ipaddr);
    my $netmaskBits = IPquadToAddr ($netmask);

    # check for class A IP address
    if (($netmaskBits & 0xffffff) == 0)
    {
	return sprintf ("%d.0.0.0/8", ($ipaddrBits >> 24) & 0xff);
    }

    # check for class B IP address
    if (($netmaskBits & 0xffff) == 0)
    {
	return sprintf ("%d.%d.0.0/16", ($ipaddrBits >> 24) & 0xff,
			                ($ipaddrBits >> 16) & 0xff);
    }

    # check for class C IP address
    if (($netmaskBits & 0xff) == 0)
    {
	return sprintf ("%d.%d.%d.0/24", ($ipaddrBits >> 24) & 0xff,
			                 ($ipaddrBits >> 16) & 0xff,
                                         ($ipaddrBits >> 8) & 0xff);
    }

    # Bummer. This subnet cannot be described in prefix notation so
    # we'll have to return the entire IP address.

    return $ipaddr;
}

#------------------------------------------------------------
# Given an IP address and netmask, the computeLocalNetworkSpec function
# computes a valid xxx.xxx.xxx.xxx/yyy.yyy.yyy.yyy specifier.
#------------------------------------------------------------

sub computeLocalNetworkSpec ($$)
{
    my ($ipaddr, $netmask) = @_;

    my $ipaddrBits  = IPquadToAddr ($ipaddr);
    my $netmaskBits = IPquadToAddr ($netmask);

    # check for all-ones netmask
    if (($netmaskBits & 0xffffffff) == 0xffffffff)
    {
	return IPaddrToQuad ($ipaddrBits);
    }

    return IPaddrToQuad ($ipaddrBits) . "/" . IPaddrToQuad ($netmaskBits);
}

#------------------------------------------------------------
# Given an IP address and netmask, the computeLocalNetworkReversed
# function computes the appropriate DNS domain field.
# 
# i.e. for an IP address of 192.168.8.4 and netmask of 255.255.255.0,
# this function will return "8.168.192.in-addr.arpa".
# 
# This string is suitable for use in BIND configuration files.
#------------------------------------------------------------

sub computeLocalNetworkReversed ($$)
{
    my ($ipaddr, $netmask) = @_;

    my $ipaddrBits  = IPquadToAddr ($ipaddr);
    my $netmaskBits = IPquadToAddr ($netmask);

    # check for class A IP address
    if (($netmaskBits & 0xffffff) == 0)
    {
	return sprintf ("%d.in-addr.arpa", ($ipaddrBits >> 24) & 0xff);
    }

    # check for class B IP address
    if (($netmaskBits & 0xffff) == 0)
    {
	return sprintf ("%d.%d.in-addr.arpa", ($ipaddrBits >> 16) & 0xff,
			                      ($ipaddrBits >> 24) & 0xff);
    }

    # check for class C IP address (treat anything smaller than a class C as a class C):

    return sprintf ("%d.%d.%d.in-addr.arpa", ($ipaddrBits >>  8) & 0xff,
			                     ($ipaddrBits >> 16) & 0xff,
			                     ($ipaddrBits >> 24) & 0xff);
}

#------------------------------------------------------------
# Given a network specification (IP address and netmask), compute
# the total number of hosts in that network, as well as the first
# and last IP addresses in the range.
#------------------------------------------------------------

sub computeHostRange ($$)
{
    my ($ipaddr, $netmask) = @_;

    my $ipaddrBits   = IPquadToAddr ($ipaddr);
    my $netmaskBits  = IPquadToAddr ($netmask);
    my $hostmaskBits = ((~ $netmaskBits) & 0xffffffff);

    my $firstAddrBits = $ipaddrBits & $netmaskBits;
    my $lastAddrBits  = $ipaddrBits | $hostmaskBits;

    my $totalHosts = 1;
    
    for ( ; $hostmaskBits; $hostmaskBits /= 2)
    {
	if (($hostmaskBits & 0x1) == 0x1)
	{
	    $totalHosts *= 2;
	}
    }

    return ($totalHosts, IPaddrToQuad ($firstAddrBits), IPaddrToQuad ($lastAddrBits));
}


#----------------------------------------------------------------------
# Utility functions related to interfaces and network zones
#----------------------------------------------------------------------






#----------------------------------------------------------------------
# Manage hosts clients lists, zones in the DNS config files and domains
#----------------------------------------------------------------------

sub print_list_dns_hosts_names ($$)
{
    my ($chroot, $domain) = @_; #($ARGV[0];
    my $hosts .= "";
    my $right_section = "n";
    open (LIST, "< $chroot/$domain") or die "Can't open the $chroot/$domain file for reading";
    while (<LIST>) {
       	chomp($_);
		$_ =~ s/\;//;
       	my @line = split (/\s+|\t+/,$_);
       	if ($line[1] eq "$domain." || $right_section eq "y") {
		$right_section = "y";
            	if ($line[0] && $line[4] && $line[0] =~ /\S+/ && $line[4] =~ /\S+/) {
                	$hosts .= $line[0]. "\=" .$line[4]. ",";
            	};
        };
    };
    close (LIST);

    $hosts =~ s/,$//;
    print "$hosts\n";
}

sub add_dns_hosts_names ($$$$)
{
    my ($chroot, $domain, $host_name, $ip) = @_; #(/var/named, design.net, sunlight, 192.168.3.45);
    my $hosts = "";
	my $conf_file = "/var/lib/naat/dns_zones";
	my $reversed_file = "";
	my $minimum = "";

	#grab information from the conf_file
	open (DNS_ZONES, "< $conf_file") or die "Can't open the $conf_file file for reading";
	while (<DNS_ZONES>) {
		chomp($_);
        my @line = split (/\s+|\t+/,$_);
		if ($line[0] && ($line[0] eq "$domain")) {
			$reversed_file = $line[1];
			$reversed_file =~ s/\.0$//;	
		};
		if ($line[8]) {	
			$minimum = $line[8];
		} else { 
			$minimum = $line[7];
		};	
	};
	close (DNS_ZONES);

	$ip =~ m/(.*)\.(.*)/;
	my $ip_suffix = $2;
	open (ADD, ">> $chroot/$domain") or die "Can't open the $chroot/$domain file for appending";
    	print ADD "$host_name\t$minimum\tIN\tA\t$ip\t\;\n";
    close (ADD);

	# append now the reversed file
	open (REVERSE, ">> $chroot/$reversed_file") or die "Can't open the $chroot/$reversed_file file for appending";
    	print REVERSE "$ip_suffix\t$minimum\tIN\tPTR\t$host_name\.$domain\.\t;\n";
    close (REVERSE);
}

sub delete_dns_hosts_names ($$$)
{
    my ($chroot, $domain, $entry) = @_; #$ARGV[0];
    my $hosts = "";
	my $conf_file = "/var/lib/naat/dns_zones";
    my $right_section = "n";
	my $reversed_file = "";
	my $managed_network = "";
	my $netmask = "";
	#grab information from the $conf_file
	open (DNS_ZONES, "< $conf_file") or die "Can't open the $conf_file file for reading";
    while (<DNS_ZONES>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_);
        if ($line[0] && ($line[0] eq "$domain")) {
            $reversed_file = $line[1];
            $reversed_file =~ s/\.0$//;
			$managed_network = $line[1];
			$netmask = $line[2];
        };
    };
    close (DNS_ZONES);

	open (DEL_old, "< $chroot/$domain") or die "Can't open the $chroot/$domain file for reading";
    open (DEL_new, "> $chroot/$domain.backup") or die "Can't open the $chroot/$domain.backup file for writing";
    while (<DEL_old>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_) ;
        if ($line[1] eq "$domain.") {
            $right_section = "y";
            print DEL_new "$_\n";
            next;
        }; 
        if ($right_section eq "y") {
            next if ($line[0] eq $entry || $line[4] eq $entry);
        };
        print DEL_new "$_\n";
    };
    close (DEL_old);
    close (DEL_new);
    rename ("$chroot/$domain","$chroot/$domain.orig");
    rename ("$chroot/$domain.backup","$chroot/$domain");
    
	#now dealing with the reversed file
	$right_section = "n";
	my $managed_network_reversed = computeLocalNetworkReversed ($managed_network, $netmask);
	open (DEL_old, "< $chroot/$reversed_file") or die "Can't open the $chroot/$reversed_file file for reading";
    open (DEL_new, "> $chroot/$reversed_file.backup") or die "Can't open the $chroot/$reversed_file.backup file for writing";
    while (<DEL_old>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_) ;
        if ($line[1] eq "$managed_network_reversed.") {
            $right_section = "y";
            print DEL_new "$_\n";
            next;
        };
        if ($right_section eq "y") {
			#print join(',',@line);
			$entry =~ m/(.*)\.(.*)/;
		    if ($2) {
				my $entry_suffix = $2;#the enrty was an IP
            	next if ($line[0] eq $entry_suffix);
			} else {
				next if ($entry && $line[4] && ($line[4] =~ m/$entry/));
			};
        };
        print DEL_new "$_\n";
    };
    close (DEL_old);
    close (DEL_new);
    rename ("$chroot/$reversed_file","$chroot/$reversed_file.orig");
    rename ("$chroot/$reversed_file.backup","$chroot/$reversed_file");
}


sub increase_serial ($$)
{ 	
	# $serial=20001215002 for Dec 15 2000 and 002 modifications on that day
    my ($chroot, $domain) = @_;
    my $serial = (localtime)[5]+1900 . (localtime)[4]+1 . (localtime)[3];
	my $conf_file = "/var/lib/naat/dns_zones";
    my $reversed_file = "";
	my $managed_network = "";
    my $netmask = "";

	#grab information from the $conf_file
    open (DNS_ZONES, "< $conf_file") or die "Can't open the $conf_file file for reading";
    while (<DNS_ZONES>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_);
        if ($line[0] && ($line[0] eq "$domain")) {
            $reversed_file = $line[1];
            $reversed_file =~ s/\.0$//;
            $managed_network = $line[1];
            $netmask = $line[2];
        };
    };
    close (DNS_ZONES);

    open (SERIAL_old, "< $chroot/$domain") or die "Can't open the $chroot/$domain file for reading";
    open (SERIAL_new, "> $chroot/$domain.orig") or die "Can't open the $chroot/$domain file for writing";
    while (<SERIAL_old>) {
        chomp($_);
        my @line = split (/\t+|\s+/,$_) ;
        if ($. == 4) { #this is the right line
            if ($line[1] =~ m/$serial/) { #we've made some modifications the same day
                $_ = "\t\t".++$line[1]." ".$line[2]." ".$line[3]." ".$line[4]." ".$line[5]." ".$line[6]." ";
            } else {
                # we replace the serial
                $_ = "\t\t".$serial."001"." ".$line[2]." ".$line[3]." ".$line[4]." ".$line[5]." ".$line[6]." ";
            };
        };
        print SERIAL_new "$_\n";
    };
    close (SERIAL_old);
    close (SERIAL_new);
    rename ("$chroot/$domain","$chroot/$domain.backup");
    rename ("$chroot/$domain.orig","$chroot/$domain");

	#now deal with the reverse file
	open (REVERSED_SERIAL_old, "< $chroot/$reversed_file") or die "Can't open the $chroot/$reversed_file file for reading";
    open (REVERSED_SERIAL_new, "> $chroot/$reversed_file.orig") or die "Can't open the $chroot/$reversed_file file for writing";
    while (<REVERSED_SERIAL_old>) {
        chomp($_);
        my @line = split (/\t+|\s+/,$_) ;
        if ($. == 5) { #this is the right line
            if ($line[1] =~ m/$serial/) { #we've made some modifications the same day
                $_ = "\t\t".++$line[1]." ".$line[2]." ".$line[3]." ".$line[4]." ".$line[5]." ".$line[6]." ";
            } else {
                # we replace the serial
                $_ = "\t\t".$serial."001"." ".$line[2]." ".$line[3]." ".$line[4]." ".$line[5]." ".$line[6]." ";
            };
        };
        print REVERSED_SERIAL_new "$_\n";
    };
    close (REVERSED_SERIAL_old);
    close (REVERSED_SERIAL_new);
    rename ("$chroot/$reversed_file","$chroot/$reversed_file.backup");
    rename ("$chroot/$reversed_file.orig","$chroot/$reversed_file");

	
}

#-----------------------------------------------------------------------------------
# Manage the zones list from the named.conf and /var/lib/naat/dns-zones file and 
# creates/remove the config files
#-----------------------------------------------------------------------------------

sub print_list_dns_zones_names_parse_named_conf ($$)
{
    my ($chroot, $domain) = @_; #($ARGV[0];
    my $conf_file = $chroot."/named.conf";
    my $zones = "";
    my $next_zone_is_reversed = "n";
    open (LIST, "< $conf_file") or die "Can't open the $conf_file file for reading";
    while (<LIST>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_);
        next if (!$line[0]);
        if (($line[0] eq "zone") && ($next_zone_is_reversed eq "n")) {
                $line[1] =~ s/\"(.*)\"/$1/g;
                if ($line[1] eq ".") {
                    $line[1] = "root";
                };
                $zones .= $line[1] .","; 
                $next_zone_is_reversed = "y";
        } elsif (($line[0] eq "zone") && ($next_zone_is_reversed eq "y")) {
                $next_zone_is_reversed = "n";
        };
    };
    close (LIST);
    
    $zones =~ s/,$//;
    print "$zones\n";
}

#deals with /var/lib/naat/dns_zones file instead of /etc/named.conf directly
sub print_list_dns_zones_names ($$)
{
    my ($conf_file, $domain) = @_; #($ARGV[0];
    my $zones = "";
    open (LIST, "< $conf_file") or die "Can't open the $conf_file file for reading";
    while (<LIST>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_);
        next if (!$line[0] || $line[0] =~m/\#/);
        $zones .= $line[0] . ",";
    };
    close (LIST);

    $zones =~ s/,$//;
    print "$zones\n";
}

sub delete_dns_zones_names ($$)
{
# we parse the /var/lib/naat/dns-zone file that I will call conf_file
    my ($conf_file, $domain) = @_; #$ARGV[0];
    open (DEL_old, "< $conf_file") or die "Can't open the $conf_file file for reading";
    open (DEL_new, "> $conf_file.backup") or die "Can't open the $conf_file.backup file for writing";
    while (<DEL_old>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_) ;
        next if ($line[0] eq $domain);
        print DEL_new "$_\n";
    };
    close (DEL_old);
    close (DEL_new);
    rename ("$conf_file","$conf_file.orig");
    rename ("$conf_file.backup","$conf_file");

# we parse the /etc/named.conf file
	my $dns_conf_file = "/etc/named.conf";
	my $two_zones_counter = 0;
	open (NAMED_CONF_old, "< $dns_conf_file ") or die "Can't open the $dns_conf_file  file for reading";
    open (NAMED_CONF_new, "> $dns_conf_file.backup") or die "Can't open the $dns_conf_file.backup file for writing";
    while (<NAMED_CONF_old>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_) ;
		#next if (!$line[0] && !$line[1]); 
        if ($line[1] && ($line[1] eq "\"".$domain."\"") || ($two_zones_counter > 0)) {
			if ($line[0] && ($line[0] eq "zone")) {
				++$two_zones_counter;
				if ($two_zones_counter == 3) {
					$two_zones_counter = 0;
					print NAMED_CONF_new "$_\n";
				}; 
			};
			next;
		};
        print NAMED_CONF_new "$_\n";
    };
    close (NAMED_CONF_old);
    close (NAMED_CONF_new);
	rename ("$dns_conf_file","$dns_conf_file.orig");
    rename ("$dns_conf_file.backup","$dns_conf_file");


}


# we parse the /var/lib/naat/dns-zone file that I will call conf_file
sub delete_dns_config_files ($$)
{
    my ($conf_file, $domain) = @_; #$ARGV[0];
	rename ("/var/named/$domain","/var/named/$domain\.deleted");
	open (CONFIG_FILE, "< $conf_file") or die "Can't open the $conf_file file for reading"; 
		while (<CONFIG_FILE>) {
			chomp($_);
			next if ($_ =~ m/\#/);
			my @line = split (/\s+|\t+/,$_) ;
			if ($line[0] = $domain) {
				my $managed_network = $line[1];
				$managed_network =~ s/\.0$//;
				rename ("/var/named/$managed_network","/var/named/$managed_network\.deleted");
			};
		};		
	close (CONFIG_FILE);
}

sub add_dns_zones_names ($$$$$$$$$) #9 parameters
{
    my ($conf_file, $domain, $managed_network, $netmask, $type, $masterIPaddress, $refresh, $retry, $expire, $minimum) =@_; #(/var/named,$ARGV[0]);
	if (!$netmask) {
        $netmask = "255.255.255.0";
    };
    if (!$type) {
        $type = "master";
    };
	if (!$masterIPaddress) {
        $masterIPaddress = "";
    };
    if (!$refresh) {
        $refresh = "28800";
    };
    if (!$retry) {
        $retry ="7200";
    };
    if (!$expire) {
        $expire ="604800";
    };
    if (!$minimum) {
        $minimum ="86400";
    };
    open (ADD, ">> $conf_file") or die "Can't open the $conf_file file for appending";
    if ($type eq "slave") {
        print ADD "$domain\t$managed_network\t$netmask\t$type\t$masterIPaddress\t$refresh\t$retry\t$expire\t$minimum\n";
    } elsif ($type eq "master"){
        print ADD "$domain\t$managed_network\t$netmask\t$type\t\t$refresh\t$retry\t$expire\t$minimum\n";
    };
    close (ADD);


	# appending to the /etc/named/conf file
	my $dns_config_file = "/etc/named.conf";
	my $reversed = computeLocalNetworkReversed ($managed_network, $netmask);
	my $managed_network_truncated = $managed_network;
    $managed_network_truncated =~ s/\.0$//;
	open (ADD_TO_NAMED_CONF, ">> $dns_config_file") or die "Can't open the $dns_config_file file for appending";
		if ($type eq "master") {
			print ADD_TO_NAMED_CONF "\nzone \"$domain\" \{\n\ttype $type\;\n\tfile \"$domain\"\;\n\}\;\n"; #print the zone section
			print ADD_TO_NAMED_CONF "zone \"$reversed\" \{\n\ttype $type\;\n\tfile \"$managed_network_truncated\"\;\n\}\;\n";
		} elsif ($type eq "slave") {
			print ADD_TO_NAMED_CONF "\nzone \"$domain\" \{\n\ttype $type\;\n\tmasters \{$masterIPaddress\;\}\;\n\tfile \"$domain\"\;\n\}\;\n"; #print the zone section
			print ADD_TO_NAMED_CONF "zone \"$reversed\" \{\n\ttype $type\;\n\tmasters \{$masterIPaddress\;\}\;\n\tfile \"$managed_network_truncated\"\;\n\}\;\n";
		};
	close (ADD_TO_NAMED_CONF);
	
}

sub update_named_conf ()
{
	my $dns_config_file = "/etc/named.conf";
	my $conf_file = "/var/lib/naat/dns_zones";
	my $reversed_file ="";
	my $managed_network = "";
	my $netmask = ""; 
	my $domain = "";
	my $type = "";
	my $masterIPaddress = "";

	#grab information from the $conf_file
    open (DNS_ZONES, "< $conf_file") or die "Can't open the $conf_file file for reading";
    while (<DNS_ZONES>) {
        chomp($_);
        my @line = split (/\s+|\t+/,$_);
        if ($line[0] && ($line[0] ne "\#")) {
            $reversed_file = $line[1];
            $reversed_file =~ s/\.0$//;
            $managed_network = $line[1];
            $netmask = $line[2];
			$domain = $line[0];
			$type = $line[3];
			if ($type eq "slave") {
				$masterIPaddress = $line[4];
			};
			
		    my $reversed = computeLocalNetworkReversed ($managed_network, $netmask);
		    my $managed_network_truncated = $managed_network;
 			$managed_network_truncated =~ s/\.0$//;
		    open (ADD_TO_NAMED_CONF, ">> $dns_config_file") or die "Can't open the $dns_config_file file for appending";
       		if ($type eq "master") {
      	    	print ADD_TO_NAMED_CONF "\nzone \"$domain\" \{\n\ttype $type\;\n\tfile \"$domain\"\;\n\}\;\n"; #print the zone section
				print ADD_TO_NAMED_CONF "zone \"$reversed\" \{\n\ttype $type\;\n\tfile \"$managed_network_truncated\"\;\n\}\;\n";
        	} elsif ($type eq "slave") {
            	print ADD_TO_NAMED_CONF "\nzone \"$domain\" \{\n\ttype $type\;\n\tmasters \{$masterIPaddress\;\}\;\n\tfile \"$domain\"\;\n\}\;\n"; #print the zone section
            	print ADD_TO_NAMED_CONF "zone \"$reversed\" \{\n\ttype $type\;\n\tmasters \{$masterIPaddress\;\}\;\n\tfile \"$managed_network_truncated\"\;\n\}\;\n";
        	};
    		close (ADD_TO_NAMED_CONF);

		};
    };
    close (DNS_ZONES);
}

sub create_dns_config_files ($$$$$$$$$)
{
    my ($conf_file, $domain, $managed_network, $netmask, $type, $masterIPaddress, $refresh, $retry, $expire, $minimum) =@_; #(/var/named,$ARGV[0]);
	if (!$netmask) {
		$netmask = "255.255.255.0";
	};
	if (!$type) {
        $type = "master";
    };
	if (!$masterIPaddress) {
        $masterIPaddress = "";
    };
    if (!$refresh) {
        $refresh = "28800";
    };
    if (!$retry) {
        $retry ="7200";
    }; 
    if (!$expire) {
        $expire ="604800";
    };  
    if (!$minimum) {
        $minimum ="86400";
    };  

	#create the /var/named/domain file
	open (ADD, "> /var/named/$domain") or die "Can't open the /var/named/$domain file for writing";
		print ADD ";BIND DUMP V8\n";
		$domain =~ m/(.*)\.(.*)/;#define $1 before dot and $2 after the dot
		my $domain_prefix = $1; 
		my $domain_suffix = $2.".";
 		print ADD "\$ORIGIN $domain_suffix\n";	# $ORIGIN net.
		print ADD "$domain_prefix\t$minimum\tIN\tSOA\t$domain\. root\.$domain\. \(\n";	#florin  86400   IN      SOA     florin.org. root.florin.org. (
		my $serial = (localtime)[5]+1900 . (localtime)[4]+1 . (localtime)[3];
		print ADD "\t\t".$serial."001 ".$refresh." ".$retry." ".$expire." ".$minimum." \)\t\;\n";
		my $i = 0;
		for ($i=0; $i <= scalar(find_listen_on_entries ())-1; $i++) {
			print ADD "\t".$minimum."\tIN\tNS\t".(find_listen_on_entries ())[$i]."\.\t\;\n";
		};
		print ADD "\t".$minimum."\tIN\tMX\t10 mail\.".$domain."\.\t\;\n";
 		print ADD "\$ORIGIN $domain\.\n";	# $ORIGIN net.
	close (ADD);

	#create the /var/named/reverse file using the managed_network argument
	my $managed_network_truncated = $managed_network;
	$managed_network_truncated =~ s/\.0$//;
    open (ADD, "> /var/named/$managed_network_truncated") or die "Can't open the /var/named/$managed_network_truncated file for writing";
        print ADD ";BIND DUMP V8\n";
		my $managed_network_reversed = computeLocalNetworkReversed ($managed_network, $netmask);
		$managed_network_reversed =~ m/([^\.])+\.(.*)/;#define $1.$2
        my $managed_network_reversed_prefix = $1;
        my $managed_network_reversed_suffix = $2;
        print ADD "\$ORIGIN $managed_network_reversed_suffix\. \n";  # $ORIGIN 2.168.192.in-addr.arpa. for example
		my $system_name = "";
		open (NETWORK, "< /etc/sysconfig/network") or die "Can't open the /etc/sysconfig/network file for reading";
			while (<NETWORK>) {
				chomp($_);
				if ( $_ =~ m/HOSTNAME/) {
        			my @line = split (/\=+|\.+/,$_);
					$system_name = $line[1];
				};
			};
		close (NETWORK);
        print ADD "$managed_network_reversed_prefix\t$minimum\tIN\tNS\t$system_name\.$domain\.\t\;\n";  #florin  86400   IN    
        print ADD "\t\t$minimum\tIN\tSOA\t$system_name\.$domain\. root\.$system_name\.$domain\.\t\(\n";  #florin  86400   IN   
		print ADD "\t\t".$serial."001 ".$refresh." ".$retry." ".$expire." ".$minimum." \)\t\;\n";
		print ADD "\$ORIGIN $managed_network_reversed\.\n";   # $ORIGIN net.
    close (ADD);
}

sub find_listen_on_entries ()	
{	
	my $dns_conf_file = "/etc/named.conf";
	open (NAMED_CONF, "< $dns_conf_file") or die "Can't open the $dns_conf_file file for reading";
	while (<NAMED_CONF>) {
		chomp($_);
		$_ =~ s/\{|\;|\}//g;
		my @line = split (/\s+|\t+/,$_);
        next if (!$line[1]);
		if ($line[1] eq "listen-on") {
			shift @line;shift @line;#remove the first 2 elements
			return (@line);
		};		
	};
	close (NAMED_CONF);
}


#-------------------------------------------------------------------
# Create the secret file, gets the secret and then removes the files
#-------------------------------------------------------------------

sub dns_dhcp_secret ()
{
    	my $secret = "";
    	my $name = "/var/lib/naat/KDHCP_UPDATER.+157+00000.key";
    	if (!-e $name) {
    		chdir("/var/lib/naat") || die "cannot cd to /var/lib/naat ($!)";
    		system("dnskeygen -H 128 -u -c -n DHCP_UPDATER > /dev/null");
    	};
	# looking for the line $3
    	open(SECRET,"</var/lib/naat/KDHCP_UPDATER.+157+00000.key") or die "Can't open the /var/lib/naat/KDHCP_UPDATER.+157+00000.key file";
    	while (<SECRET>) {
        	$secret = $_;
        	last if $. == 1;
    	};
    	chomp ($secret);
    	if ($. != 1) {
        	die "Didn't find line 1 in the /var/lib/naat/KDHCP_UPDATER.+157+00000.key file\n";
    	};
    	close(SECRET);
    	$secret =~ s/(\S+\s+){6}//;
    return ($secret);
}

#--------------------------------
#--------------------------------
# SQUIDGUARD management section #
#--------------------------------
#--------------------------------

sub print_list_squidGuard_db_file ($)
{
    my ($db_file) = @_; 
    my $entries .= "";
    my $separator = "";
    open (LIST, "< $db_file") or die "Can't open the $db_file file for reading";
	while (<LIST>) {
		if (($_ !~ /^\s+$/) && ($_ !~ /^\#/)) {
			$_ =~ s/<\s*//g;
			$_ =~ s/\s*>//g;
			$_ =~ s/\(/\{/g;
			$_ =~ s/\)/\}/g;
			$_ =~ s/\|/_pipe_/g;
			$_ =~ s/,/+/g;
			$_ =~ s/ /_/g;
			chomp $entries;
			$entries .= $separator.$_;
			$separator = ",";
		};
   	};
    close (LIST);
    print "$entries";
}

sub add_to_list_squidGuard_db_file ($$)
{
    my ($db_file, $entry) = @_;
    
    # create the directory if needed
    my $db_file_directory = dirname ($db_file);
    system ("mkdir -p $db_file_directory");
	$entry =~ s/_pipe_/\|/g;
	$entry =~ s/_/ /g;
	$entry =~ s/\{/\(/g;
	$entry =~ s/\}/\)/g;
        my @line = split (/\+/,$entry);
	my $separator ="";
	$entry ="";
	for (my $i=0; $i <= scalar(@line)-1;++$i) {
		$entry .= $separator.$line[$i];
		if ($db_file =~ m/phrase/i) {
			$separator = ">,<";
		};
	};
	if ($db_file =~ m/phrase/i) {
    		$entry = "<".$entry.">";
	};
    open (ADD, ">> $db_file") or die "Can't open the $db_file file for appending";
	print ADD "$entry\n";
    close (ADD);
}

sub delete_squidGuard_db_file_entry ($$)
{
    my ($db_file, $entry) = @_;
	$entry =~ s/_pipe_/\|/g;
	$entry =~ s/\{/\(/g;
	$entry =~ s/\}/\)/g;
	$entry =~ s/_/ /g;
	my @line = split (/\+/,$entry);
	my $to_delete ="yes";

    open (DEL_old, "< $db_file") or die "Can't open the $db_file file for reading";
    open (DEL_new, "> $db_file.backup") or die "Can't open the $db_file.backup file for writing";
	while (<DEL_old>) {
    	chomp($_);
		$to_delete ="yes";
		if (($_ !~ /^\s*$/) && ($_ !~ /^\#/)) {
#			$_ =~ s/<\s*//g;
#			$_ =~ s/\s*>//g;
#			$_ =~ s/,/ /g;
			foreach my $key (@line) {
				if ($_ !~ m/$key/i) {	
					$to_delete = "no";
				};
			};
		} else {
        	print DEL_new "$_\n";
		};
        next if ($to_delete eq "yes");
        print DEL_new "$_\n";
    };
    close (DEL_old);
    close (DEL_new);
    rename ("$db_file","$db_file.orig");
    rename ("$db_file.backup","$db_file");
}

sub print_list_squidGuard_expr_file ($)
{
    my $temp ="";
    my $i = my $length = 0;
    my ($expr_file) = @_;
    open (LIST, "< $expr_file") or die "Can't open the $expr_file file for reading";    
	while (<LIST>) {
        chomp($_);
        my @line = split (/\|/,$_);
		$length = scalar(@line);
		for ($i =0; $i <= $length-1; $i++) {
			$temp .= $line[$i] . ",";
        };
	};
    close (LIST);
    $temp =~ s/(\(|\)|,$)//g;
    print "$temp\n";
}

sub add_to_list_squidGuard_expr_file ($$)
{
    my ($expr_file, $entry) = @_;
	my $found = my $length = my $i = 0;
	my $temp = "";
	
    # create the directory if needed
    my $expr_file_directory = dirname ($expr_file);
    system ("mkdir -p $expr_file_directory");

	open (ADD, "< $expr_file") or die "Can't open the $expr_file file for reading";
    while (<ADD>) {
        chomp($_);
        my @line = split (/\|/,$_);
        $length = scalar(@line);
		$line[0] =~ s/(\(|\))//g; #chop off the ( and )
		$line[$length-1] =~ s/(\(|\))//g; #chop off the ( and )
        for ($i =0; $i <= $length-1; $i++) {
			if ($line[$i] eq $entry) { 			# the entry exists already
				$found = 1;
			};
            $temp .= $line[$i] . ",";
        };
    };
    close (ADD);

	$temp =~ s/(^,|,$)//g;		# chop off the first and the last ","
	if ($found == 0) { 
		if ($length > 0) {
			$temp .= "|".$entry;
		} else {
			$temp .= $entry;
		};
	};
	$temp =~ s/,/\|/g;
	open (ADD_new, "> $expr_file.new") or die "Can't open the $expr_file file for writing";
    	print ADD_new "(".$temp.")\n";
	close (ADD_new);
    rename ("$expr_file","$expr_file.orig");
    rename ("$expr_file.new","$expr_file");
}

sub delete_squidGuard_expr_file_entry ($$)
{
    my ($expr_file, $entry) = @_;
	my $found = my $length = my $i = 0;
	my $temp = "";
	
    # create the directory if needed
    my $expr_file_directory = dirname ($expr_file);
    system ("mkdir -p $expr_file_directory");

	open (DEL, "< $expr_file") or die "Can't open the $expr_file file for reading";
    while (<DEL>) {
        chomp($_);
        my @line = split (/\|/,$_);
        $length = scalar(@line);
        for ($i =0; $i <= $length-1; $i++) {
			if ($i == 0) {
				$line[0] =~ s/(\(|\))//g;
			} elsif ($i == $length-1) {
				$line[$length-1] =~ s/(\(|\))//g;
			}; 
			if ($line[$i] eq $entry) {
					next;
			#		$found = 1;
			};
            $temp .= $line[$i] . ",";
        };
    };
    close (DEL);

	$temp =~ s/,$//;
	$temp =~ s/,/\|/g;
	open (DEL_new, "> $expr_file.new") or die "Can't open the $expr_file file for writing";
    	if ($length-1 > 0) {
			print DEL_new "(".$temp.")\n";
		};
	close (DEL_new);
    rename ("$expr_file","$expr_file.orig");
    rename ("$expr_file.new","$expr_file");
}


#-------------------------------
# Shorewall management section #
#-------------------------------
sub shorewall_hash ()
{ 
	my $shorewall = {
		
		'hosts' => {
			'1' => 'zone',
			'2' =>  {  
				'host' => {
					'1' => 'interface',
					'2' => 'ip',
				},
			},
			'3' => 'options',
		},

		'zones' => {
			'1' => 'zone',
			'2' => 'display',
			'3' => 'comments',
		},

		'interfaces' => {
			'1' => 'zone',
			'2' => 'interface',
			'3' => 'broadcast',
			'4' => 'options',
		},

		'policy' => {
			'1' => 'client',
			'2' => 'server',
			'3' => 'policy',
			'4' => 'log_level',
		},

		'masq' => {
			'1' => {
				'interface' => {
				'1' => 'interface',
				'2' => 'ip',
				},
			},
			'2' => 'subnet',
			'3' => 'address',
		},

		'nat' => {
			'1' => 'external',
			'2' => 'interface',
			'3' => 'internal',
			'4' => 'all',
			'5' => 'local',
		},

		'proxyarp' => {
			'1' => 'address',
			'2' => 'interface',
			'3' => 'external',
			'4' => 'haveroute',
		},

		'tunnels' => {
			'1' => 'type',
			'2' => 'zone',
			'3' => 'gateway',
			'4' => 'gateway_zone',
		},

		'rules' => {
			'1' => {
				'result' => {
					'1' => 'result',
					'2' => 'log_level',
				},
			},
			'2' => {
				'client' => {
					'1' => 'client_zone',
					'2' => 'client_ip',
				},
			},
			'3' => {
				'server' => {
					'1' => 'server_zone',
					'2' => 'server_ip',
					'3' => 'server_port',
				},
			},
			'4' => 'proto',
			'5' => 'port',
			'6' => 'client_port',
			'7' => {
				'address' => {
					'1' => 'address',
					'2' => 'ip_snat',
				},
			},
		},

		'tcrules' => {
			'1' => 'mark',
			'2' => 'source',
			'3' => 'dest',
			'4' => 'proto',
			'5' => 'port',
			'6' => 'client_port',
		},

		'tos' => {
			'1' => {
				'client' => {
					'1' => 'client_zone',
					'2' => 'client_ip',
				},
			},
			'2' => {
				'server' => {
					'1' => 'server_zone',
					'2' => 'server_ip',
				},
			},
			'3' => 'proto',
			'4' => 'client_port',
			'5' => 'server_port',
			'6' => 'tos',
		},

		'blacklist' => {
			'1' => 'IPs',
		},

        };

return($shorewall);
	
}

sub shorewall_infos  
{                       
        my @args = @_;
	my $shorewall = $args[0];
         
	my $match = ""; 

	if ( $args[1] =~ /position/) {
		my $string = "";
		foreach my $key ( keys %{$shorewall->{$args[2]}}) { 
			
			if (!ref($shorewall->{$args[2]}->{$key}) && ($shorewall->{$args[2]}->{$key} eq $args[3])) {
				$string .= $key;
				$match = "yes";
				return ($match,$string);
			} elsif (ref($shorewall->{$args[2]}->{$key})) {
				foreach my $key1 ( keys %{$shorewall->{$args[2]}->{$key}}) {
					my ($a, $b) =  shorewall_infos ($shorewall->{$args[2]}->{$key}, "position", "$key1", "$args[3]");
					$match = $a if ($a);
					$string = $key . $b if ($b); 
					if ($match && ($match eq "yes")) { 
						return ($string) 
					};
				}
			}
		}

	} elsif ( $args[1] =~ /label/) {
		my $string = "";
		$args[3] =~ m/([^+])(.*)/;
		my $first_part = $1;
		my $second_part = $2;
	
		if (ref($shorewall->{$args[2]}->{$first_part})) {
			foreach my $key ( keys %{$shorewall->{$args[2]}->{$first_part}}) {
				if ($second_part) {	
					return ($shorewall->{$args[2]}->{$first_part}->{$key}->{$second_part});
				} else {
					return($key);
				}
			}
		} else { 
			return ($shorewall->{$args[2]}->{$first_part});
		}	

	} elsif ($args[1] =~ /coloumns/) {
		foreach my $key ( sort { "$a$b" =~ /^\d+$/ ? $b <=> $a : $a cmp $b } keys %{$shorewall->{$args[2]}}) { 
			return ($key);
			last;
		};
	}
			
}

#------------------------------------------------------------
# Given a domain name such as foo.bar.com, generate the
# LDAP base name "dc=foo,dc=bar,dc=com".
#------------------------------------------------------------

sub ldapBase ($)
{
    my ($domainName) = @_;
    $domainName =~ s/\./,dc=/g; 
    return "dc=" . $domainName;
}

#------------------------------------------------------------
# Run command in background after a specified delay.
#------------------------------------------------------------

sub backgroundCommand ($@)
{
    my ($delaySec, @command) = @_;

    # now would be a good time to flush output buffers, so the partial
    # buffers don't get copied

    $| = 1;
    print "";

    # create child process
    my $pid = fork;

    # if fork failed, bail out
    die "Cannot fork: $!" unless defined ($pid);

    # If fork succeeded, make parent process return immediately.
    # We are not waiting on the child, so it will become a zombie
    # process when it completes. However, this subroutine is only
    # intended for use by the e-smith signal-event program, which
    # doesn't run very long. Once the parent terminates, the zombie
    # will become owned by "init" and will be reaped automatically.

    return if ($pid);

    # detach ourselves from the terminal
    setsid || die "Cannot start a new session: $!";

    # change working directory
    chdir "/";

    # clear file creation mask
    umask 0;

    # close STDIN, STDOUT, and STDERR
    close STDIN;
    close STDOUT;
    close STDERR;

    # reopen stderr, stdout, stdin
    open (STDIN, '/dev/null');
    open (STDOUT, '>>/tmp/e-smith-bg.log');
    open (STDERR, '>&STDOUT');

    # make child wait for specified delay.
    sleep $delaySec;

    # execute command
    exec { $command[0] } @command;
}

#============================================================
# Low-level password-changing utilities. These utilities each
# change passwords for a single underlying password database,
# for example /etc/passwd, /etc/smbpasswd, etc.
#============================================================

#----------------------------------------
# Set Unix password
#----------------------------------------

sub setUnixPassword ($$)
{
    my ($username, $password) = @_;
    my $success = 0;

    my $autoPasswdProg = '/usr/bin/autopassword';
    
    my $pid = open(RES, "-|"); # perldoc perlipc

    if ($pid) # parent
    {
	while (<RES>)
	{
	    if (/^\+RESULT Password changed successfully.$/)
	    {
		$success = 1;
	    }
	}
    }
    else
    {
	exec $autoPasswdProg, $username, $password, $password;
	die "exec of '$autoPasswdProg $username pass pass' failed: $!";
    }

    if ($success == 0)
    {
	die "Failed to set Unix password for account $username.\n";
    }

    return 1;  # success
}

# setUnixPasswordRequirePrevious is left as an exercise for the reader :-)

#----------------------------------------
# Set Unix password but require previous password
#----------------------------------------

sub setUnixPasswordRequirePrevious ($$$)
{
    my ($username, $oldpassword, $newpassword) = @_;
    my $success = 0;

    my $autoPasswdProg = '/usr/bin/autopassword';
    
    my $pid = open(RES, "-|"); # perldoc perlipc

    if ($pid) # parent
    {
	while (<RES>)
	{
	    if (/^\+RESULT Password changed successfully.$/)
	    {
		$success = 1;
	    }
	}
    }
    else
    {
	exec $autoPasswdProg,
	    $username, $oldpassword, $newpassword, $newpassword;
	die "exec of '$autoPasswdProg $username old new new' failed: $!";
    }

    if ($success == 0)
    {
	die "Failed to set Unix password for account $username.\n";
    }

    return 1;  # success
}

#----------------------------------------
# Set Samba password
#----------------------------------------

sub setSambaPassword ($$)
{
    my ($username, $password) = @_;

    #----------------------------------------
    # then set the password
    #----------------------------------------

    my $smbPasswdProg = '/usr/bin/smbpasswd';
    # see perldoc perlipc (search for 'Safe Pipe Opens')
    my $pid = open(DISCARD, "|-");

    if ($pid) # parent
    {
	print DISCARD "$password\n$password\n";
	close(DISCARD) || die "Child exited early.";
    }
    else # child
    {
	my $retval = system("$smbPasswdProg -a -s $username >/dev/null");
	($retval/256) &&
	    die "Failed to set Samba password for account $username.\n";
	exit 0;
    }

    my $retval = system("$smbPasswdProg -e -s $username >/dev/null");
    ($retval/256) &&
	die "Failed to enable Samba account $username.\n";

    return 1;  # success
}

#----------------------------------------
# Cancel Samba password
#----------------------------------------

sub cancelSambaPassword ($)
{
    my ($username) = @_;

    #----------------------------------------
    # Gordon Rowell <gordonr@e-smith.net> June 7, 2000
    # We really should maintain old users, which would mean we can just use
    # smbpasswd -d, but the current policy is to remove them. If we are
    # doing that (see below), there is no need to disable them first.
    #----------------------------------------
#    my $discard = `/usr/bin/smbpasswd -d -s $username`;
#    if ($? != 0)
#    {
#	die "Failed to disable Samba account $username.\n";
#    }

    #----------------------------------------
    # Delete the smbpasswd entry. If we don't, re-adding the same
    # username will result in a mismatch of UIDs between /etc/passwd
    # and /etc/smbpasswd
    #----------------------------------------
    # Michael Brader <mbrader@stoic.com.au> June 2, 2000
    # We have a locking problem here.
    # If two copies of this are run at once you could see your entry reappear
    # Proposed solution (file locking):

    # If we do a 'use Fcntl, we'll probably get the locking constants
    # defined, but for now:

    # NB. hard to test

    my $LOCK_EX = 2;
    my $LOCK_UN = 8;

    my $smbPasswdFile = '/etc/smbpasswd';

    open(RDWR, "+<$smbPasswdFile") || # +< == fopen(path, "r+",...
	die "Cannot open file $smbPasswdFile: $!\n";

    my $nolock = 1;
    my $attempts;
    for ($attempts = 1;
	 ($attempts <= 5 && $nolock);
	 $attempts++)
    {
	if (flock(RDWR, $LOCK_EX))
	{
	    $nolock = 0;
	}
	else
	{
	    sleep $attempts;
	}
    }

    $nolock && die "Could not get exclusive lock on $smbPasswdFile\n";

    my $outputString = '';
    while (<RDWR>)
    {
	(/^$username:/) || ($outputString .= $_);
    }

    # clear file and go to beginning
    truncate(RDWR, 0) || die "truncate failed"; # not 'strict' safe why???
    seek(RDWR, 0, 0) || die "seek failed";
    print RDWR $outputString;
    flock(RDWR, $LOCK_UN)
	|| warn "Couldn't remove exclusive lock on $smbPasswdFile\n";
    close RDWR || die "close failed";

    chmod 0600, $smbPasswdFile;

    return 1;  # success
}

#----------------------------------------
# Generate password for LDAP
#----------------------------------------

sub genLdapPassword ()
{
    # Pick a password at random. We take three random lowercase
    # letters and append the process ID. We return this to the caller,
    # and also write it to the file /etc/openldap/ldap.pw. Typically this
    # will be called while creating the LDAP config files, then at
    # runtime, the LDAP commands that require passwords will look in
    # this file to find out the password.

    srand (time);

    my $i;
    my $password = "";

    for ($i=0; $i < 3; $i++)
    {
        $password .= chr (int (rand (26)) + 97);
    }
    $password .= $$;

    open (WR, ">/etc/openldap/ldap.pw") || die "Could not write LDAP password file.\n";
    print WR "$password\n";
    close WR;

    chmod 0600, "/etc/openldap/ldap.pw";

    return $password;
}

#============================================================
# High-level password-changing utilities. These utilities
# each change passwords for a single e-smith entity (system,
# user or ibay). Each one works by calling the appropriate
# low-level password changing utilities.
#============================================================

#----------------------------------------
# Set the e-smith system password
#----------------------------------------

sub setUnixSystemPassword ($)
{
    my ($password) = @_;
    
    setUnixPassword   ("root",  $password);
    setUnixPassword   ("admin", $password);
}

sub setServerSystemPassword ($)
{
    my ($password) = @_;

    setSambaPassword  ("admin", $password);
}

#----------------------------------------
# Set e-smith user password
#----------------------------------------

sub setUserPassword ($$)
{
    my ($username, $password) = @_;

    setUnixPassword   ($username, $password);
    setSambaPassword  ($username, $password);
}

#----------------------------------------
# Set user password - require previous password
#----------------------------------------

sub setUserPasswordRequirePrevious ($$$)
{
    my ($username, $oldpassword, $newpassword) = @_;

    # if old password is not valid, this statement will call "die"
    setUnixPasswordRequirePrevious ($username, $oldpassword, $newpassword);

    # if we get this far, the old password must have been valid
    setSambaPassword  ($username, $newpassword);
}

#----------------------------------------
# Cancel user password. This is called when a user is deleted from the
# system. We assume that the Unix "useradd/userdel" programs are
# called separately. Since "userdel" automatically removes the
# /etc/passwd entry, we only need to worry about the /etc/smbpasswd
# entry.
#----------------------------------------

sub cancelUserPassword ($)
{
    my ($username) = @_;

    cancelSambaPassword ($username);
}

#----------------------------------------
# Set ibay password
#----------------------------------------

sub setIbayPassword ($$)
{
    my ($ibayname, $password) = @_;

    setUnixPassword ($ibayname, $password);
}

sub serviceNumber ($$)
{
    my ($name, $protocole) = @_;
    open SERVICE, "</etc/services" or return $name;
    while (<SERVICE>) {
	if (/^$name\s+(\d+)\/$protocole/)
	{
	    close SERVICE;
	    return $1;
	}
    }
    return $name;
}

#------------------------------------------------------------
# build hash from list with items
# To have specific value $hash_table{ $item_key}{ $value_key }
#-----------------------------------------------------------
sub build_hash_from_list($)
{
    my $list = shift @_;

    my %h;

    if ( $list !~ /\S+/ ) {
	return %h;
    }

    # split the List around character ","
    my @items = split (/,/,$list);

    foreach (@items) {

	my ($item, $attr_list) = /(\S+)\s*\((.*)\)/;
	# item      : a
	# attr_list : b=1 c=1 d=2

	if ($attr_list) {
	    my @vals = split ' ', $attr_list;

# problem with map with the compiled C version
#	    $h{$item} = { map { split '\s*=\s*'; } @vals };
# Doing this instead... :-(
	    foreach my $pair (@vals) {
		my ($attribute,$value) = split '\s*=\s*', $pair;
		$h{$item}{$attribute} = $value;
	    }

	} else {
	    $h{$_} = {}; 
	}
    }

    return %h;
}


#------------------------------------------------------------
# No module clean-up code required.
#------------------------------------------------------------

END
{
}

#------------------------------------------------------------
# Return "1" to make the import process return success.
#------------------------------------------------------------

1;
