#! /usr/bin/perl -W
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2001 Mandrakesoft
# Author Sylvain de Tilly
#
# $Id: add_rule.pl,v 1.4 2002/12/04 15:26:16 philippe Exp $

# This script add to a list rules (See /var/lib/naa/configuration) a new rules
# define by options passed.
# There is two way to create rules :
#  - Use tmp var pre-define by another script.
#  - Passe all options.
#
# If protocol options isn't defined, second arguments will be considered
# as new protocol to opend.




use strict;

use vars qw($opt_server $opt_client $opt_result $opt_protocol $opt_tmpvars);
use Getopt::Long;

use lib qw(/usr/lib/naat/);
use backend::util;


my $Usage = <<"End_of_Usage;";
Usage: add_rule.pl transaction_name rules_list_to_add  port1,port2,...
   With the options :
          -s|--server   "server_zone"
          -c|--client   "server_client"
          -r|--result   "result" (DROP|REJECT|ACCEPT)
          -p|--protocol "protocol"

   With tmp vars :
Usage: add_rule.pl transaction_name rules_list_to_add --tmpvars
          -t|--tmpvars  Use tmp vars"
End_of_Usage;


my $NULL_ITEM = "---\n"; # Defined in config-wrapper.pl

# First get the transaction name (FIRST STEP IN ALL SCRIPTS)
my $transaction_name = shift @ARGV if (@ARGV && $ARGV[0] !~ /^-/);
my $rules_list_to_add = shift @ARGV if (@ARGV && $ARGV[0] !~ /^-/);
my $rules;

# print STDERR "$transaction_name et $rules_list_to_add\n";

if (! $transaction_name ) {
    $transaction_name = "$$";
    # initialize a backend transaction
    system ("config-wrapper.pl $transaction_name -i");
}

# config-wrapper convention (/usr/bin/config-wrapper.orig.pl)


my $get_opt_result = GetOptions( "tmpvars", "server=s", "client=s",
				 "protocol=s", "result=s" );

#print STDERR "tmpvars => $opt_tmpvars\n";


# Check if we must use configuration file tmp's vars.
if ( $opt_tmpvars ) {
  my $server_zone    = `config-wrapper.pl $transaction_name --get TmpServer`;
  my $client_zone    = `config-wrapper.pl $transaction_name --get TmpClient`;
  my $nat            = `config-wrapper.pl $transaction_name --get TmpNat`;
  my $protocol       = `config-wrapper.pl $transaction_name --get TmpProtocol`;
  my $server_forward = `config-wrapper.pl $transaction_name --get TmpServerForward`;
  my $result         = `config-wrapper.pl $transaction_name --get TmpResult`;
  my $services_list  = `config-wrapper.pl $transaction_name --get TmpPort`;

  my @services = split( ',', $services_list );

  # remove all '\n' at the end of the concerned string
  chop( $server_forward, $server_zone, $client_zone, $nat, $protocol,
	$result, @services );



  foreach ( @services ) {
    $rules .= "server $server_zone client $client_zone result $result ";

    if ( $nat ne $NULL_ITEM && $server_forward ne $NULL_ITEM ) {
      $rules .= "nat_ext $nat server_forward $server_forward ";
    }

    if ( $protocol eq $NULL_ITEM ) {
      $rules .= "protocol $_,";
    }

    # Miss just the protocol (which must be diffrent than udp and tcp
    else {
      $rules .= "protocol $protocol  port $_,";
    }
  }

  if ( defined $rules ) {
    chop( $rules );		# skip the last ','
  # No rule defined.
  }
  else {
    print STDERR $Usage;
    exit ( 1 );
  }
}

# Check if there is at least one arg and the compulsory options
elsif ( $ARGV[ 0 ] && $opt_server && $opt_client && $opt_result ) {
  foreach ( split( ',', $ARGV[ 0 ] ) ) {
    $rules .= "server $opt_server client $opt_client result $opt_result ";

    # Check if protocol is define
    # yes => $ARGV[ 0 ] is ports
    if ( $opt_protocol ) { $rules .= "protocol $opt_protocol port $_,"; }
    else { $rules .= "protocol $_,"; }
  }

  chop( $rules ); # skip the last ','
}

# Bad options
else {
  print STDERR $Usage;
  exit ( 1 );
}


# print STDERR "--------------------------------------------------------------------------------\n";
# print STDERR "ADD_LIST.PL\n\tvariable => $rules_list_to_add\n\trules => $rules\n";
# print STDERR "--------------------------------------------------------------------------------\n";

# Exec config-wrapper.pl
`config-wrapper.pl $transaction_name --listadd $rules_list_to_add $rules`;
