#!/bin/sh
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 
##
##  gentestcrt -- Create self-signed test certificate
##  (C) 2001 Jean-Michel Dault <jmdault@mandrakesoft.com> and Mandrakesoft
##  Based on cca.sh script by Ralf S. Engelschall
##  2002 Modified by Florin Grad <florin@mandrakesoft.com>

mkdir -p /etc/freeswan/ipsec.d/{cacerts,private,crls}
freeswan_path="/etc/freeswan"
location="/etc/freeswan/all_certificates"
private_dir="/etc/freeswan/ipsec.d/private"
cert_dir="/etc/freeswan/ipsec.d"
cacerts_dir="/etc/freeswan/ipsec.d/cacerts"
crls="/etc/freeswan/ipsec.d/crls"
mkdir -p $location
touch $freeswan_path/index.txt
#   external tools
openssl="/usr/bin/openssl"

do_ca() {

echo ""
echo ""

if [ ! -e ./server.crt -a ! -e ./server.key ];then 
	echo "Will create server.key and server.crt in `pwd`"
else
	echo "server.key and server.crt already exist, dying"
	exit
fi

echo ""

    echo "${T_MD}INITIALIZATION${T_ME}"

    echo ""
    echo "${T_MD}Generating custom Certificate Authority (CA)${T_ME}"
    echo "______________________________________________________________________"
    echo ""
    echo "${T_MD}STEP 1: Generating RSA private key for CA (2048 bit)${T_ME}"
    cp /dev/null $location/ca.rnd
    echo '01' >$location/ca.ser
    echo "$randfiles"
    if [ ".$randfiles" != . ]; then
        $openssl genrsa -rand $randfiles -out $private_dir/ca.key 2048
    else
        $openssl genrsa -out $private_dir/ca.key 2048
    fi
    if [ $? -ne 0 ]; then
        echo "cca:Error: Failed to generate RSA private key" 1>&2
        exit 1
    fi
    echo "______________________________________________________________________"
    echo ""
    echo "${T_MD}STEP 2: Generating X.509 certificate signing request for CA${T_ME}"

    $openssl req -config /etc/freeswan/config_file_ca -new -key $private_dir/ca.key -out $location/ca.csr
    if [ $? -ne 0 ]; then
        echo "cca:Error: Failed to generate certificate signing request" 1>&2
        exit 1
    fi
    echo "______________________________________________________________________"
    echo ""
    echo "${T_MD}STEP 3: Generating X.509 certificate for CA signed by itself${T_ME}"
    $openssl x509 -extfile /etc/freeswan/config_file_ca -req -days 3650 -signkey $private_dir/ca.key -in $location/ca.csr -out $cacerts_dir/ca.crt
    if [ $? -ne 0 ]; then
        echo "cca:Error: Failed to generate self-signed CA certificate" 1>&2
        exit 1
    fi
    echo "______________________________________________________________________"
    echo ""
    echo "${T_MD}RESULT:${T_ME}"
    $openssl verify $cacerts_dir/ca.crt
    if [ $? -ne 0 ]; then
        echo "cca:Error: Failed to verify resulting X.509 certificate" 1>&2
        exit 1
    fi
    $openssl x509 -text -in $cacerts_dir/ca.crt
    $openssl rsa -text -in $private_dir/ca.key
    $openssl ca -config $freeswan_path/config_file_ca -gencrl -cert $cacerts_dir/ca.crt -out $crls/crl.crt

				
}

do_server() {
    echo "${T_MD}CERTIFICATE GENERATION${T_ME}"
    user="server"

    echo ""
    echo "${T_MD}Generating custom USER${T_ME} [$user]"
    echo "______________________________________________________________________"
    echo ""
    echo "${T_MD}STEP 5: Generating RSA private key for USER (2048 bit)${T_ME}"
    if [ ".$randfiles" != . ]; then
        $openssl genrsa -rand $randfiles -out $private_dir/$user.key 2048
    else
        $openssl genrsa -out $private_dir/$user.key 2048
    fi
    if [ $? -ne 0 ]; then
        echo "cca:Error: Failed to generate RSA private key" 1>&2
        exit 1
    fi
    echo "______________________________________________________________________"
    echo ""
    echo "${T_MD}STEP 6: Generating X.509 certificate signing request for USER${T_ME}"
    
    $openssl req -config /etc/freeswan/config_file_ca -new -key $private_dir/$user.key -out $location/$user.csr
    if [ $? -ne 0 ]; then
        echo "cca:Error: Failed to generate certificate signing request" 1>&2
        exit 1
    fi
    #rm -f /etc/freeswan/config_file_ca
    echo "______________________________________________________________________"
    echo ""
    echo "${T_MD}STEP 7: Generating X.509 certificate signed by own CA${T_ME}"
    $openssl x509 -extfile /etc/freeswan/config_file_server_x509 -days 3650 -CAserial $location/ca.ser -CA $cacerts_dir/ca.crt -CAkey $private_dir/ca.key -in $location/$user.csr -req -out $cert_dir/$user.crt
    if [ $? -ne 0 ]; then
        echo "cca:Error: Failed to generate X.509 certificate" 1>&2
        exit 1
    fi
    caname="`$openssl x509 -noout -text -in $cacerts_dir/ca.crt |\
             grep Subject: | sed -e 's;.*CN=;;' -e 's;/Em.*;;'`"
    username="`$openssl x509 -noout -text -in $cert_dir/$user.crt |\
               grep Subject: | sed -e 's;.*CN=;;' -e 's;/Em.*;;'`"
#    echo "Assembling PKCS#12 package"
#    $openssl pkcs12 -export -in $user.crt -inkey $user.key -certfile ca.crt -name "$username" -caname "$caname" -out $user.p12
    echo "______________________________________________________________________"
    echo ""
    echo "${T_MD}RESULT:${T_ME}"
    $openssl verify -CAfile $cacerts_dir/ca.crt $cert_dir/$user.crt
    if [ $? -ne 0 ]; then
        echo "cca:Error: Failed to verify resulting X.509 certificate" 1>&2
        exit 1
    fi
    $openssl x509 -text -in $cert_dir/$user.crt
    $openssl rsa -text -in $private_dir/$user.key
    mkdir -p /etc/freeswan/ipsec.d/{cacerts,private,crls}
    mv $private_dir/$user.key $private_dir/`grep -r CN /etc/freeswan/config_file_ca |sed -e 's|[^.]*=||g'`.key
    cp $cert_dir/$user.crt $cert_dir/`grep -r CN /etc/freeswan/config_file_ca |sed -e 's|[^.]*=||g'`.crt
    if [ ! -e $freeswan_path/x509cert.der ]; then
	    mv $cert_dir/$user.crt $freeswan_path/x509cert.der
    else 	    
	    echo "The file $freeswan_path/x509cert.der already exists"
	    rm -rf $cert_dir/$user.crt
    fi

				
}

do_usage() {
    echo "Unknown arg $i";
		echo "usage: gentestcrtca|gentestcrtserver|gentestcrtwindows yourhostname" >&2
    exit 1  
}

do_export_windows() {
	if [ -z "$1" ]; then
        	echo "You forgot the hostname. Example: gentestcrtwindows mystation.mydomain.org" 1>&2
	        exit 0
	else
#    echo "Assembling PKCS#12 package"
$openssl pkcs12 -export -in $cert_dir/$1.crt -inkey $private_dir/$1.key -certfile $cacerts_dir/ca.crt -out $cert_dir/$1.p12
	fi
}

#   some optional terminal sequences
case $TERM in
    xterm|xterm*|vt220|vt220*)
        T_MD=`echo dummy | awk '{ printf("%c%c%c%c", 27, 91, 49, 109); }'`
        T_ME=`echo dummy | awk '{ printf("%c%c%c", 27, 91, 109); }'`
        ;;
    vt100|vt100*)
        T_MD=`echo dummy | awk '{ printf("%c%c%c%c%c%c", 27, 91, 49, 109, 0, 0); }'`
        T_ME=`echo dummy | awk '{ printf("%c%c%c%c%c", 27, 91, 109, 0, 0); }'`
        ;;
    default)
        T_MD=''
        T_ME=''
        ;;
esac

#   find some random files
#   (do not use /dev/random here, because this device 
#   doesn't work as expected on all platforms)
randfiles=''
for file in /var/log/messages /var/log/syslog \
            /boot/vmlinuz /var/log/secure \
            /etc/hosts /etc/resolv.conf; do
    if [ -f $file ]; then
        if [ ".$randfiles" = . ]; then
            randfiles="$file"
        else
            randfiles="${randfiles}:$file"
        fi
    fi
done

#echo "${T_MD}maketestcrt -- Create self-signed test certificate${T_ME}"
#echo "(C) 2001 Jean-Michel Dault <jmdault@mandrakesoft.com> and Mandrakesoft"
#echo "Based on cca.sh script by Ralf S. Engelschall"
#echo ""

grep -q -s DUMMY server.crt && mv server.crt server.crt.dummy
grep -q -s DUMMY server.key && mv server.key server.key.dummy

case $0 in
				*gentestcrtca)
				echo "in CA mode: $0"
				do_ca
				;;
				*gentestcrtserver)
				echo "in server mode: $0"
				do_server
				;;
				*gentestcrtwindows)
				echo -ne "export in pkcs12 (for Windows) format: $0"
				do_export_windows $1
				;;
				*)
				echo "nom: $0"
				case $2 in
								-ca)
								do_ca
								;;
								-server)
								do_server
								;;
								*)
								echo "unknown"
								do_usage
				esac
				;;
esac
