#!/usr/bin/perl
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2001 Mandrakesoft
# Author Renaud Chaillat <rchaillat@mandrakesoft.com>
#
# $Id: interfaces.pl,v 1.6 2003/12/02 16:12:05 florin Exp $
#

use lib qw(/usr/lib/libDrakX);
use lib qw(/usr/lib/naat/backend);

use common;
use detect_devices;
use any;
use modules;
use network::network;
use network::ethernet;
use network::isdn;
use network::modem;
use ethernet_info;

# Usage: interfaces.pl [trans_name] device_type

# exit code:
# 0       : interface found
# -1 (255): none found
# 1       : argument error

# First get the transaction name (FIRST STEP IN ALL SCRIPTS)
my $transaction_name = shift @ARGV if ( scalar(@ARGV) == 2 );

my $device_type = shift @ARGV;
exit (1) unless defined $device_type;

$::isStandalone = 1;




#------------------------------------------------------------------------------
# Retrieve info about network cards
#------------------------------------------------------------------------------
# Prints :
#  (string) $cards : comma separated list of interfaces 
#                    (i.e.  "eth0 (Driver=... Mac=...),eth1 (Driver=... Mac=...) ...")
#
# TODO: manual config for some isa cards
#
if ($device_type =~ /net/) {
    my $netc = {};

    print STDERR "Interfaces: trying to find network devices...\n";

    $^W=0;

    # Load appropriate network modules and fill (modules::)%conf hash table
    run_program::run("depmod","-A"); # to avoid error messages on STDOUT...
    #any::setup_thiskind_backend ('net', 1, 0, sub { });

    # Retrieve the list of all ethernet cards
    my @all_cards = conf_network_card_backend ("", $netc);

    $^W=1;

    ($#all_cards == -1) and print STDERR "No ethernet network adapter has been detected on your system."
	                and exit (2);

    # Update /etc/modules.conf and /lib/modules/<kernel version>/modules.dep
    # TODO: instead of writing conf now, get vars and trigger events later ?
    print STDERR "Updating modules.conf...\n";
    $^W=0;
    modules::write_conf(""); # TODO: give the prefix found in /etc/naat/backend.conf
    $^W=1;
    print STDERR "Rebuilding modules dependecies...\n";
    $^W=0;
    run_program::run("depmod","-a");
    $^W=1;

    # Find mac addresses for each interface and build the output for calling script
    my ($cards, $info);
    my $config_string = "";
    foreach (@all_cards) {
	# Find mac address of current ethernet interface
	my $interface = shift @$_; # get interface name
	my $module    = shift @$_; # get module name
	my $mac_addr  = backend::ethernet_info::get_interface_info ($interface);
	chomp $mac_addr;
	chomp $interface;
	$module ||= "unknown";
	chomp $module;
	
	# keep only the driver name, and append the mac address of the interface
	$info .= "$interface (Driver=$module Mac=$mac_addr),";

	# build the "set" string to pass to config-wrapper
	$config_string .= "-s \u${interface}Known true ";
	$config_string .= "-s \u${interface}Driver $module ";
	$config_string .= "-s \u${interface}Mac $mac_addr ";

    }

    if ( $config_string ne "" && $transaction_name ) {
	system ("config-wrapper.pl $transaction_name $config_string ");
    }

    # Output for calling script
    $info =~ s/,$//;
    if ($info =~ /\S+/) {
	print "$info \n";
    } else {
	print "---\n";
	exit (-1);
    }

}


#------------------------------------------------------------------------------
# Retrieve info about one *detected* isdn card
#------------------------------------------------------------------------------
# Prints :
#    (string) $isdn_card : isdn card description
#                          (i.e description (ID#... Driver#... Vendor#... CardType#... Type#...) ) 
#
elsif ($device_type =~ /isdn/) {

    print STDERR "Interfaces: trying to find an isdn device...\n";
    
    my %isdn_card = ();
    $^W=0;
    network::isdn_detect_backend (\%isdn_card);
    $^W=1;

    # print info
    if ($isdn_card{description} =~ /\S+/) {
	$isdn_card{description} =~ s/ /%20/g;
	print "$isdn_card{description} (Id=$isdn_card{id} Driver=$isdn_card{driver} Vendor=$isdn_card{vendor} CardType=$isdn_card{card_type} Type=$isdn_card{type})\n";

	# Write the info in the current transaction for easier treatment
	system ("config-wrapper.pl", "$transaction_name","--set", "ISDNDeviceType", $isdn_card{type},
		"--set", "ISDNDriver",$isdn_card{driver});
    } else {
	print "---\n";
	exit (-1);
    }
}


#------------------------------------------------------------------------------
# Retrieve info about modems
#------------------------------------------------------------------------------
# Prints :
#
elsif ($device_type =~ /modem/) {

    print STDERR "Interfaces: trying to find a modem...\n";
    
    my %modem = ();
    $^W=0;
    modem_detect_backend (\%modem);
    $^W=1;

    # print info
    if ($modem{device} =~ /\S+/) {
	print "$modem{device} \n";
    } else {
	print "---\n";
	exit (-1);
    }

}







