#!/usr/bin/perl
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2001 Mandrakesoft
# Author Renaud Chaillat <rchaillat@mandrakesoft.com>
#
# $Id: isdn-cards.pl,v 1.3 2003/10/01 12:24:58 florin Exp $
#

use lib qw(/usr/lib/libDrakX);
use lib(qw(/usr/lib/libDrakX/network);
use isdn;
use netconnect;

# ARGS (without a transaction name)
# No comand line arguments             : print isdn cards list
# 1 arg  (--info)                      : get info abot the isdn card found in the current transaction file
# 2 args (--info <cardname>)           : print all infos  for card <cardname>
# 3 args (--info <cardname> <varname>) : print <varname> value for card <cardname>

# First get the transaction name (FIRST STEP IN ALL SCRIPTS)
my $transaction_name = shift @ARGV if ( (scalar(@ARGV)==1  && $ARGV[0] !~ /^-/ ) ||
					(scalar(@ARGV) >=2  && $ARGV[1] =~ /^-/)    );


#
# Prints known isdn cards list in a comma separated string
#
if ( 0 == scalar (@ARGV) ) {
    my $cards_list = join ",", netconnect::isdn_get_list();

    $cards_list = encrypt ($cards_list);

    print $cards_list."\n";
    exit (0);
}

#
# Prints info about a card
#  
my $option = shift @ARGV;
if ( $option eq "--info" ) {
    if ( 2 == scalar (@ARGV) ) {
	print_isdn_card_single_info (@ARGV);
    }
    else {
	print_isdn_card_info (@ARGV);
    }
}


#
# Get all informations about one isdn card
#
# Args   : (string) cardname : name of the card 
#
# Prints : comma separated list of card infos
#          (i.e. description#... , irq#... , card#... , type#... , mem#... , io#... )
sub 
print_isdn_card_info {

    my $cardname = shift @_;

    if (!$transaction_name) { 
	$transaction_name = "$$";
	# initialize a backend transaction
	system ("config-wrapper.pl", "$transaction_name", "-i");
    }


    if (!$cardname) { # read it in the current transaction file
	$cardname = `config-wrapper.pl $transaction_name --get ISDNCardDescription`;

	if (!$cardname || $cardname !~ /\S+/ || $cardname =~ /---/) { 
		print STDERR "No info found in transaction $transaction_name 
						for variable ISDNCardDescription\n";
	    exit (-1);
	}
    }

    chomp $cardname; # beware of newlines...
    $cardname =~ s/%20/ /g;

    print STDERR "ISDN Cards: Retrieving info for card ${cardname}\n";
#    dumpValue([netconnect::isdn_get_info($cardname)]);

    my %infos = %{netconnect::isdn_get_info($cardname)};
    my $output = "";
    while (my ($key, $value) = each(%infos)) {
	$output .= "$key=$value,";
    }
    $output =~ s/,$//;
    if ($output eq "") {
	print STDERR "No info found for card **${cardname}**\n";
	print "---\n"; 
	exit (-1);
    }
    print "$output \n";

    # Write the info in the current transaction for easier treatment
    system ("config-wrapper.pl", "$transaction_name","--set", "ISDNCardType", $infos{card},
	    "--set", "ISDNDeviceType",$infos{type},"--set","ISDNMem",$infos{mem},
	    "--set","ISDNIo",$infos{io},"--set","ISDNIrq",$infos{irq});


    if ( $transaction_name eq "$$" ) {
	# cleanup the transaction
	system ("config-wrapper.pl $transaction_name -a");
    }

}

#
# Get a specific information about one isdn card
#
# Args   : (string) cardname : name of the card 
#          (string) varname  : name of the parameter we want information about
#
# Prints : a single string (value of the parameter specified)
sub 
print_isdn_card_single_info {
    my $cardname = shift @_;
    my $isdn_varname = shift @_;

    print STDERR "ISDN Cards: Retrieving info for card ${cardname}\n";

    my %infos = %{netconnect::isdn_get_info($cardname)};

    # Map the ISDNVarName to a field name in the hash table
    # and print the value of this field
    for ($isdn_varname) {
	/ISDNCardDescription/ and do { defined $infos{description} && print "$infos{description}\n"; };
	/ISDNIrq/             and do { defined $infos{irq} && print "$infos{irq}\n"; };
        /ISDNCardType/        and do { defined $infos{card} && print "$infos{card}\n"; }; # isa/pci
	/ISDNDeviceType/      and do { defined $infos{type} && print "$infos{type}\n"; };      # number used by the driver
	/ISDNMem/             and do { defined $infos{mem} && print "$infos{mem}\n"; };
        /ISDNIo/              and do { defined $infos{io} && print "$infos{io}\n"; };
    }
}


#this sub replace many characters by their %XX counterparts
sub encrypt
{
  my $result = shift;
  $result =~ s/%/%25/g; # this one should be the first !!!
  $result =~ s/\t/%09/g;
  $result =~ s/\n/%13/g;
  $result =~ s/\r/%10/g;
  $result =~ s/\n/%13/g;
  $result =~ s/ /%20/g;
  $result =~ s/!/%21/g;
  $result =~ s/\"/%22/g;
  $result =~ s/\#/%23/g;
  $result =~ s/$/%24/g;
  $result =~ s/&/%26/g;
  $result =~ s/\'/%27/g;
  $result =~ s/\(/%28/g;
  $result =~ s/\)/%29/g;
  $result =~ s/\*/%2A/g;
  $result =~ s/\+/%2B/g;
#not here  $result =~ s/,/%2C/g;
  $result =~ s/-/%2D/g;
  $result =~ s/\./%2E/g;
  $result =~ s!/!%2F!g;

  return $result;
}
