#!/usr/bin/perl
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2001 Mandrakesoft
# Author Renaud Chaillat <rchaillat@mandrakesoft.com>
#
# $Id: isdn-providers.pl,v 1.2 2002/12/04 15:26:16 philippe Exp $
#

use lib qw(/usr/lib/libDrakX);
use netconnect;

# ARGS (without a transaction name)
# No comand line arguments                 : print isdn known providers list
# 1 arg  (--info)                          : get info abot the provider found in the current transaction file
# 2 args (--info <providername>)           : print all infos  for provider <providername>
# 3 args (--info <providername> <varname>) : print <varname> value for provider <providername>

# First get the transaction name (FIRST STEP IN ALL SCRIPTS)
my $transaction_name = shift @ARGV if ( (scalar(@ARGV)==1 && $ARGV[0] !~ /^-/) || 
					(scalar(@ARGV)>=2 && $ARGV[1] =~ /^-/)   );



#
# Prints known isdn providers list in a comma separated string
#
if ( 0 == scalar (@ARGV) ) {
    my $providers_list = join ",", netconnect::read_providers_backend('/etc/naat/isdndb.net','');

    $providers_list = encrypt ($providers_list);

    print $providers_list."\n";
    exit (0);
}

#
# Prints info about a provider
#  
my $option = shift @ARGV;
if ( $option eq "--info" ) {
    if ( 2 == scalar (@ARGV) ) {
	print_isdn_provider_single_info (@ARGV);
    }
    else {
	print_isdn_provider_info (@ARGV);
    }
}


#
# Get all informations about one isdn provider
#
# Args   : (string) cardname : name of the provider 
#
# Prints : comma separated list of provider infos
#           (i.e. name#... , phone_out#... , domain#... , dns1#... , dns2#... )
sub 
print_isdn_provider_info {
    my $providername = shift @_;

	if (!$transaction_name) { 
	    $transaction_name = "$$";
	    # initialize a backend transaction
	    system ("config-wrapper.pl", "$transaction_name", "-i");
	}

    if (!$providername) { # read it in the current transaction file
	$providername = `config-wrapper.pl $transaction_name --get ISDNProvider`;

	if (!$providername || $providername !~ /\S+/) { 
		print STDERR "No provider name found \n";
	    exit (-1);
	}
    }

	chomp $providername; # beware of newlines

    print STDERR "ISDN Providers: Retrieving info for provider ${providername}\n";

    my %isdn = ();
    my %netc = ();
    netconnect::get_info_providers_backend(\%isdn,\%netc,$providername,'/etc/naat/isdndb.net','');
    my $output = "";
    my $request = "";
    defined ($isdn{user_name})   and do { $output .= "name#$isdn{user_name},"; $request .= " --set ISDNLogin $isdn{user_name}";};
    defined ($isdn{phone_out})   and do { $output .= "phone_out#$isdn{phone_out},";  $request .= " --set ISDNProviderPhone $isdn{phone_out}";};
    defined ($netc{DOMAINNAME2}) and do { $output .= "domain#$netc{DOMAINNAME2},";  $request .= " --set ISDNProviderDomain $netc{DOMAINNAME2}";};
    defined ($netc{dnsServer2})  and do { $output .= "dns1#$netc{dnsServer2},";  $request .= " --set ISDNProviderDNS1 $netc{dnsServer2}";};
    defined ($netc{dnsServer3})  and do { $output .= "dns2#$netc{dnsServer3},";  $request .= " --set ISDNProvidereDNS2 $netc{dnsServer3}";};
    $output =~ s/,$//;

    print "$output \n";


    # Write the info in the current transaction for easier treatment
    system ("config-wrapper.pl $transaction_name $request");

    if ( $transaction_name eq "$$" ) {
	# cleanup the transaction
	system ("config-wrapper.pl $transaction_name -a");
    }

}

#
# Get a specific information about one isdn provider
#
# Args   : (string) provider : name of the provider
#          (string) varname  : name of the parameter we want information about
#
# Prints : a single string (value of the parameter specified)
sub 
print_isdn_provider_single_info {
    my $providername = shift @_;
    my $isdn_varname = shift @_;

    print STDERR "ISDN Providers: Retrieving info for provider ${providername}\n";

    my %isdn = ();
    my %netc = ();
    netconnect::get_info_providers_backend(\%isdn,\%netc,$providername,'/etc/naat/isdndb.net','');

    # Map the ISDNVarName to a field name in the hash table
    # and print the value of this field
    for ($isdn_varname) {
	/ISDNLogin/           and do { defined $isdn{user_name}   && print "$isdn{user_name}\n"; };
	/ISDNProviderPhone/   and do { defined $isdn{phone_out}   && print "$isdn{phone_out}\n"; };
        /ISDNProviderDomain/  and do { defined $netc{DOMAINNAME2} && print "$netc{DOMAINNAME2}\n"; };
	/ISDNProviderDNS1/    and do { defined $netc{dnsServer2}  && print "$netc{dnsServer2}\n"; }; 
	/ISDNProviderDNS2/    and do { defined $netc{dnsServer3}  && print "$netc{dnsServer3}\n"; };
    }
}


#this sub replace many characters by their %XX counterparts
sub encrypt
{
  my $result = shift;
  $result =~ s/%/%25/g; # this one should be the first !!!
  $result =~ s/\t/%09/g;
  $result =~ s/\n/%13/g;
  $result =~ s/\r/%10/g;
  $result =~ s/\n/%13/g;
  $result =~ s/ /%20/g;
  $result =~ s/!/%21/g;
  $result =~ s/\"/%22/g;
  $result =~ s/\#/%23/g;
  $result =~ s/$/%24/g;
  $result =~ s/&/%26/g;
  $result =~ s/\'/%27/g;
  $result =~ s/\(/%28/g;
  $result =~ s/\)/%29/g;
  $result =~ s/\*/%2A/g;
  $result =~ s/\+/%2B/g;
#not here  $result =~ s/,/%2C/g;
  $result =~ s/-/%2D/g;
  $result =~ s/\./%2E/g;
  $result =~ s!/!%2F!g;

  return $result;
}
