#! /usr/bin/perl -W
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2000,2001 Mandrakesoft
# Author Sylvain de Tilly
#
#
# This script sort and print rules by options passed to it.
#
# $Id: rules_sorting.pl,v 1.5 2002/12/04 15:26:16 philippe Exp $

use strict;

use lib qw(/usr/lib/naat/);
use backend::util;

use Getopt::Long;
# Due to use strict, we must declare opts variables
#use vars qw($opt_s $opt_f $opt_r $opt_c $opt_n $opt_p $opt_k $opt_h);
use vars qw( $opt_server $opt_client $opt_protocol $opt_forward $opt_rules $opt_return_key $opt_nat $opt_help);

# prototype
sub make_to_return_string ($$$;$);

my $Usage =<<"End_of_Usage;";
Usage: rules_sorting.pl transaction
  --key|-k      return hash keys
  --forward|-f  search in forward list
  --rules|-r    search in rules list
  --nat|-n      print if NAT is used
  --server|-s   [server_zone] print all rules which destination zone is "server_zone"
  --client|-c   [client_zone] print all rules which source zone is "client_zone"
  --protocol|-p [proto] print all rules which protocol is proto
     proto = other for protocol differrents than udp|tcp
  --help|-h     print this help
End_of_Usage;



# First get the transaction name (FIRST STEP IN ALL SCRIPTS)
my $transaction_name = shift @ARGV if (@ARGV && $ARGV[0] !~ /^-/);


if (! $transaction_name ) {
  $transaction_name = "$$";
  # initialize a backend transaction
  system ("config-wrapper.pl $transaction_name -i");
}


# The String finaly returned
my $to_write_naat = "";

# Value used to designed rules in configuration file.
my $RULES=`config-wrapper.pl $transaction_name --get FirewallRules`;
my $FORWARD=`config-wrapper.pl $transaction_name --get FirewallAlwaysForwardServices`;
my $POLICY=`config-wrapper.pl $transaction_name --get FirewallPolicy`;

# The rule hash table.
my %rules_hash = ();
my %forward_hash = ();
my %policy_hash = backend::util::build_hash_from_list( $POLICY );

# Call the lon getopt
my $return_getopt = GetOptions( 'server=s','client=s', 'protocol=s',
				'forward', 'rules','key', 'nat', 'help' );

if ( !$return_getopt ) {
  print STDERR $Usage;
  exit ( 1 );
}

# Options -f if you want to search in the forwardServices list
if ( $opt_forward ) {
  %forward_hash = backend::util::build_hash_from_list( $FORWARD );
}
# Options -r if you want to search in the rulesServices list
if ( $opt_rules ) {
  %rules_hash = backend::util::build_hash_from_list( $RULES );
}


# Exit if there is too much arguments or if -h is given.
if ( $ARGV[ 0 ] || $opt_help) {
  print STDERR "$ARGV[ 0 ] et $opt_help\n";
  print STDERR $Usage;
  exit( 1 );
}

# print STDERR <<"End_Options;";
# OPTIONS : - protocol $opt_protocol
# 	    - forward $opt_forward
# 	    - rules $opt_rules
# 	    - server $opt_server
# 	    - client $opt_client
# 	    - nat $opt_nat
# 	    - return_key $opt_return_key
# 	    - help $opt_help
# End_Options;


# The string to write.
my $to_write = "";


# First looking in general rules
foreach ( keys %policy_hash ) {
  if ( ( $policy_hash{ $_ }{ "result" } &&
	 $policy_hash{ $_ }{ "result" } eq "ACCEPT" ) &&
       ( !$opt_server || $policy_hash{ $_ }{ "client" } eq $opt_server ||
	 $policy_hash{ $_ }{ "client" } eq "all" ) &&
       ( !$opt_client || $policy_hash{ $_ }{ "server" } eq $opt_client ||
	 $policy_hash{ $_ }{ "server" } eq "all" )
     ) {
    print "all\n";
    # If There is general rules, we can go out.
    exit( 0 );
  }
}

# We want to search in the FirewallRule list
if ( $opt_rules ) {
  # There is no general rule so we search rule in special rule.
  foreach (keys %rules_hash ) {
    # if server_zone is query and the same of the current rule.
    if ( ( $rules_hash{ $_ }{ "result" } &&
	   $rules_hash{ $_ }{ "result" } eq "ACCEPT" ) &&
	 ( !$opt_server || $rules_hash{ $_ }{ "server" } eq $opt_server ) &&
	 ( !$opt_client || $rules_hash{ $_ }{ "client" } eq $opt_client ) ) {


#      print STDERR "Des truc\n\t$rules_hash{ $_ }{ client }\n\t$rules_hash{ $_ }{ server }\n\t$rules_hash{ $_ }{ protocol }\n\t$rules_hash{ $_ }{ port }\n\t$rules_hash{ $_ }{ nat_ext }\n\tkey => $_\n";

      if ( !$opt_protocol ) {
	if ( $rules_hash{ $_ }{ "protocol" } ne "udp" &&
	     $rules_hash{ $_ }{ "protocol" } ne "tcp" ) {
	  make_to_return_string( \%rules_hash, "$_", "protocol" );
	}
	else {
	  make_to_return_string( \%rules_hash, "$_", "port", "protocol" );
	}
      }
      elsif ( ( $opt_protocol eq "udp" || $opt_protocol eq "tcp" ) &&
		$rules_hash{ $_ }{ "protocol" } eq $opt_protocol ) {
	make_to_return_string( \%rules_hash, "$_", "port");
      }
      elsif ( $opt_protocol ne "udp" && $opt_protocol ne "tcp" &&
		$rules_hash{ $_ }{ "protocol" } ne "udp" &&
		$rules_hash{ $_ }{ "protocol" } ne "tcp" ) {
	make_to_return_string( \%rules_hash, "$_", "protocol" );
      }
    }
  }
}
# Forward here so no nat ...
if ( $opt_forward ) {
  foreach (keys %forward_hash ) {
    if ( $forward_hash{ $_ }{ "protocol" } ne "udp" &&
	 $forward_hash{ $_ }{ "protocol" } ne "tcp" ) {
      make_to_return_string( \%forward_hash, "$_", "protocol" );
    } else {
       make_to_return_string( \%forward_hash, "$_", "port", "protocol" );
    }
  }
}

chop( $to_write );    # skip the last ','
print "$to_write\n";


sub make_to_return_string ($$$;$)
{
  my $hash_ref = shift @_;
  my %hash = %$hash_ref;
  my $key = shift @_;
  my $first_to_write = shift @_;
  my $second_to_write = shift @_ if $_[ 0 ];

#   print STDERR<<"End"
#     We have somethings to write. The proof :
# 	hash{ $key } => $hash{ $key }{ nat_ext }
# 	key => $key
# 	first_to_write => $first_to_write
# 	second_to_write => $second_to_write
# 	return_key => $return_key
# 	nat => $nat
#   End

  # the return_key flag is 1, we do it...
  if( $opt_return_key ) {
    $to_write .= "$key ";
  }

  else {
    $to_write .= $hash{ $key }{ $first_to_write };
    if ( $second_to_write ) {
      $to_write .= "($hash{ $key }{ $second_to_write })";
    }

    # If nat selected, print the ip address
    if ( $opt_nat && $hash{ $key }{ "nat_ext" } ) {
      $to_write .= "   $hash{ $key }{ nat_ext }";
    }
  }

  $to_write .= ",";
}
