#!/usr/bin/perl -w
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2001 Mandrakesoft
# Author Florin Grad <florin@mandrakesoft.com>
# 

use strict;
use lib qw(/usr/lib/naat);
use backend::util;

# First get the transaction name (FIRST STEP IN ALL SCRIPTS)
my $transaction_name = shift @ARGV if ( @ARGV && $ARGV[1] !~ /^-/ );

my $Usage =<<"End_of_Usage;";

Usage:	shorewall_manage.pl [trans_name] zones_file < -l | -lt | -a entries | -d entry | -g item>
	shorewall_manage.pl [trans_name] zones_file < --list | --listtable | --add entries | --delete entry | --(g|s)et_item item >

--list|-l	Displays the list of entries (Zones, Displays, Comments)

--listtable|-lt	Displays the list of entries in table format

--add|-a	Add an entry at the end of the requested files, before the last comment
		One needs to different arguments according to the desired file:
		- zones: zone name, display, comments
		- interfaces: zone, interface, broadcast, options
		- policy: client, server, policy, log_level
		- masq: interface, subnet
		- nat: external, interface, internal, all_interfaces, local  

		EXAMPLE: shorewall_manage.pl /etc/shorewall/zones -a dmz DMZ "Demilitarized zone" 

--delete|-d	Deletes the coresponding entry from zones file 
		EXAMPLE: shorewall_manage.pl /etc/shorewall/zones -d dmz

--get_item|-g 	Displays the required item value  
		
		EXAMPLE: shorewall_manage.pl /etc/shorewall/interfaces -g local broadcast

--set_item|-s 	Sets the required item value  
		
		EXAMPLE: shorewall_manage.pl /etc/shorewall/zones -s net comments "The Internet"

End_of_Usage;

# arg0 : list | add | delete

# Check if there are at least 2 arguments given
if ( scalar (@ARGV) le 1 ) {
    print STDERR $Usage;
    exit(3);
}

if ( $ARGV[1] eq "--list" || $ARGV[1] eq "-l") {
	backend::util::print_list_shorewall_file($ARGV[0]);
} elsif ( $ARGV[1] eq "--listtable" || $ARGV[1] eq "-lt") {
	backend::util::print_list_table_shorewall_file($ARGV[0]);
} elsif ( $ARGV[1] eq "--add" || $ARGV[1] eq "-a") {
	backend::util::delete_entry ($ARGV[0], "#LAST");
	backend::util::add_entry ($ARGV[0], $ARGV[2], $ARGV[3], $ARGV[4], $ARGV[5], $ARGV[6]);
	backend::util::add_entry ($ARGV[0], "\#LAST LINE - ADD YOUR ENTRIES ABOVE THIS ONE - DO NOT REMOVE","" ,"","","");
} elsif ( $ARGV[1] eq "--delete" || $ARGV[1] eq "-d") {
	backend::util::delete_entry ($ARGV[0], $ARGV[2]);
} elsif ( $ARGV[1] eq "--get_item" || $ARGV[1] eq "-g") {
	backend::util::get_item ($ARGV[0], $ARGV[2], $ARGV[3], $ARGV[4]); 
} elsif ( $ARGV[1] eq "--set_item" || $ARGV[1] eq "-s") {
	backend::util::set_item ($ARGV[0], $ARGV[2], $ARGV[3], $ARGV[4], $ARGV[5]); 
} else {
print STDERR $Usage;
};
