#! /usr/bin/perl -W
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

# Copyright (C) 2000,2001 Mandrakesoft
# Author Sylvain de Tilly
#
# $Id: swap_rules_to_tmp_vars.pl,v 1.3 2002/12/04 15:26:16 philippe Exp $
#
# This script store specific rule (defined by options) in configuration's tmp vars.
# When the rule to swap is find, the script exit.

use strict;

use lib qw(/usr/lib/naat/);
use backend::util;

use Getopt::Long;
use vars qw($opt_server $opt_client $opt_nat $opt_result $opt_service $opt_protocol $opt_rules $opt_help);



# Prototype
sub key_to_print($);

my $Usage =<<"End_of_Usage;";
Usage: swap_rules_to_tmp_vars.pl trans_name [options]
 Options describe the rule you want to swap in tmp vars.
    --server "server zone" The server zone.
    --client "client zone" The client zone
    --nat "IP address"     Rule use NAT port forwarding with
    			   "IP address" as virtual IP.
    --service "service"    The port used
    --result "result"      The result of the rule (ACCEPT|REJECT|DROP)
    --protocol "protocol"  The protocol used
    --rules "list"         The rule list used
    --help                 Display this message
End_of_Usage;


my $getopt_return = &GetOptions( "service=s", "server=s", "client=s", "nat=s", "protocol=s", "result=s", "help" );

if ( $opt_help ) {
  print STDERR $Usage;
  exit( 1 );
}

# First get the transaction name (FIRST STEP IN ALL SCRIPTS)
my $transaction_name = shift @ARGV if ( scalar(@ARGV) );

# Used by the frontend inputrules.naat (Not very clean :-( )
my $service = shift @ARGV if ( scalar(@ARGV) );
my $nat_ext = shift @ARGV if ( scalar(@ARGV) );

#print "service => $service\t nat_ext => $nat_ext\n";

if (! $transaction_name ) {
  $transaction_name = "$$";
  # initialize a backend transaction
  system ("config-wrapper.pl $transaction_name -i");
}



my $condition = 0;
my $condition_ok = 0;


# The list rule we want to store in it
#my $list_rule = shift @ARGV if (@ARGV && $ARGV[0] !~ /^-/);
my $list_rule = "FirewallRules";
my $RULES=`config-wrapper.pl $transaction_name --get $list_rule`;

my %rules_hash = backend::util::build_hash_from_list( $RULES );


if ( defined $opt_service )  { $condition++; }
if ( defined $opt_server )   { $condition++; }
if ( defined $opt_client )   { $condition++; }
if ( defined $opt_nat )      { $condition++; }
if ( defined $opt_protocol ) { $condition++; }
if ( defined $opt_result )   { $condition++; }

#print STDERR "condition => $condition\n";

#print STDERR "\tclient => $client_zone \n\tserver => \n\tnat_ext => $rules_hash{ $_ }{ nat_ext }\n\tport => $rules_hash{ $_ }{ port }\n";


foreach ( keys %rules_hash ) {

  $condition_ok = 0;
  if ( defined $opt_server   &&  defined $rules_hash{ $_ }{ "server" }   && $rules_hash{ $_ }{ "server" } eq $opt_server )     { $condition_ok++; }
  if ( defined $opt_client     &&  defined $rules_hash{ $_ }{ "client" }   && $rules_hash{ $_ }{ "client" } eq $opt_client )       { $condition_ok++; }
  if ( defined $opt_nat      &&  defined $rules_hash{ $_ }{ "nat_ext" }  && $rules_hash{ $_ }{ "nat_ext" } eq $opt_nat )       { $condition_ok++; }
  if ( defined $opt_service &&  defined $rules_hash{ $_ }{ "port" }     && $rules_hash{ $_ }{ "port" } eq $opt_service )     { $condition_ok++; }
  if ( defined $opt_protocol &&  defined $rules_hash{ $_ }{ "protocol" } && $rules_hash{ $_ }{ "protocol" } eq $opt_protocol ) { $condition_ok++; }
  if ( defined $opt_result   &&  defined $rules_hash{ $_ }{ "result" }   && $rules_hash{ $_ }{ "result" } eq $opt_result )     { $condition_ok++; }

  if ( $condition_ok >= $condition ) {
    if ( defined $rules_hash{ $_ }{ "server" } ) {
      `config-wrapper.pl $transaction_name --set TmpServer $rules_hash{ $_ }{ "server" }`;
    }
    if ( defined $rules_hash{ $_ }{ "client" } ) {
      `config-wrapper.pl $transaction_name --set TmpClient $rules_hash{ $_ }{ "client" }`;
    }
    if ( defined $rules_hash{ $_ }{ "nat_ext" } ) {
      `config-wrapper.pl $transaction_name --set TmpNat $rules_hash{ $_ }{ "nat_ext" }`;
    }
    if ( defined $rules_hash{ $_ }{ "protocol" } ) {
      `config-wrapper.pl $transaction_name --set TmpProtocol $rules_hash{ $_ }{ "protocol" }`;
    }
    if ( defined $rules_hash{ $_ }{ "server_forward" } ) {
      `config-wrapper.pl $transaction_name --set TmpServerForward $rules_hash{ $_ }{ "server_forward" }`;
    }
    if ( defined $rules_hash{ $_ }{ "result" } ) {
      `config-wrapper.pl $transaction_name --set TmpResult $rules_hash{ $_ }{ "result" }`;
    }
    # We could have more than one port for indentic (except port) rule.
    if ( defined $rules_hash{ $_ }{ "port" } ) {
      `config-wrapper.pl $transaction_name --set TmpPort $rules_hash{ $_ }{ "port" }`;
    }
    # Clean this rule
    `config-wrapper.pl $transaction_name --set  TmpKey $_`;
    exit( 0 );
  }
}






